function [tf, cf, pr] = compute_behav_scores()
%COMPUTE_BEHAV_SCORES computes recall rates and organization by subject
%
% INPUTS
%
% none
%
% OUTPUTS
%
% tf: temporal factor, per subject
% cf: category factor, per subject
% pr: probability of recall, per subject

% By James Kragel, 2021
% Distributed under Open Source BSD License

subs = ident_mscript_subs;

enc_pow_dir = '../../data/hdf5_encoding/';
rec_pow_dir = '../../data/hdf5_retrieval/';

tf = nan(1, length(subs));
cf = nan(1, length(subs));
pr = nan(1, length(subs));

% build a distance matrix for the categories to work with dist_mat
for s = 1:length(subs)
    
    if exist(['data\' subs{s} '_data.mat'],'file')
        load(['data\' subs{s} '_data.mat'],'data');
    else
        enc_h5file = [enc_pow_dir ...
            subs{s} '_pow.hdf5'];
        
        rec_h5file = [rec_pow_dir ...
            subs{s} '_pow.hdf5'];
        
        data = h5todata(enc_h5file, rec_h5file);
        save(['data\' subs{s} '_data.mat'],'data');
    end
    
    % build dist mat from data.pres_itemnos
    
    tmp = data.pres_itemnos;
    tmp(tmp<=0)=nan;
    
    % ignore incomplete trials or practice trials
    ig_idx = any(isnan(tmp),2) | any(tmp<0,2);
    data.pres_itemnos(ig_idx,:)=nan;
    data.rec_itemnos(ig_idx,:)=nan;
    data.recalls(ig_idx,:)=nan;
    
    
    data.pres_itemnos = tmp;
    data.rec_itemnos(data.rec_itemnos==0)=nan;
    
    [ui, ia] = unique(tmp);
    try
        uc = data.pres.categoryNum(ia);
    catch
        uc = data.pres.category_num(ia);
    end
    
    nan_idx = isnan(ui);
    ui(nan_idx) = [];
    uc(nan_idx) = [];
    
    dist_mat = nan(length(ui));
    for i = 1:length(ui)
        for j = 1:length(ui)
            if uc(j) == uc(i)
                dist_mat(ui(i),ui(j)) = 1;
            else
                dist_mat(ui(i),ui(j)) = 0;
            end
        end
    end
    
    % requires behavioral toolbox
    tf(s) = temp_fact(data.recalls, data.subject, data.listLength);
    
    cf(s) = dist_fact(data.rec_itemnos, data.pres_itemnos, data.subject, ...
        dist_mat);
    
    pr(s) = p_rec(data.recalls, data.subject, data.listLength);
end


end

function data = h5todata(enc_h5file, rec_h5file)
%H5TODATA converts events from h5 format to matlab data structure

% get the fields from each h5
info = h5info(enc_h5file);
fieldnames = {info.Groups(1).Datasets.Name};
n_events = info.Groups(1).Datasets(1).Dataspace.Size;
for e = 1:n_events
    for f = 1:length(fieldnames)
        if strcmp(fieldnames{f},'stim_params')
            continue
        end
        
        tmp = h5read(enc_h5file, ...
            ['/events/' fieldnames{f}], e, 1);
        
        
        if iscell(tmp)
            tmp = deblank(char(tmp));
        end
        
        enc_events(e).(fieldnames{f}) = tmp;
    end
end

% get the fields from each h5
info = h5info(rec_h5file);
fieldnames = {info.Groups(1).Datasets.Name};
n_events = info.Groups(1).Datasets(1).Dataspace.Size;
for e = 1:n_events
    for f = 1:length(fieldnames)
        if strcmp(fieldnames{f},'stim_params')
            continue
        end
        tmp = h5read(rec_h5file, ...
            ['/events/' fieldnames{f}], e, 1);
        
        if iscell(tmp)
            tmp = deblank(char(tmp));
        end
        rec_events(e).(fieldnames{f}) = tmp;
    end
end

events = [enc_events rec_events];

if strcmp(enc_events(1).subject,'R1290M') % issues with itemno for this subj
    [~,~,ib] = unique({events.item});
    for k = 1:length(events)
        events(k).itemno = ib(k);
    end
end


for k = 1:length(events)
    try
        events(k).itemno = events(k).item_num;
        events(k).item = events(k).item_name;
    end
end

data = FRdata(events, 'list');

end

