function [dist, bp_shared, contact_name] = compute_elec_dists(pow_dir, ...
    subject, pairs_dir)
% ELEC_DISTS computes the distances between contacts for a subject and
% whether they are in a shared bipolar pair
%
% INPUTS
%
% pow_dir: string - path that contains power hdf5s for each subject
% subject: string - subject identifier
% pairs_dir: string - directory pointing to pairs.json for data recorded on
%                     the ens system
%
% OUTPUTS
%
% dist: distance between electrodes
% bp_shared: logical array indicated whether two contacts are in the same
%            bipolar pair
% contact_name: labels for the contacts
%
% By James Kragel, 2021
% Distributed under Open Source BSD License

info = h5info([pow_dir '/' subject '_pow.hdf5']);
gnames = {info.Groups.Name};
has_tal_struct = any(strcmp('/tal_struct',gnames));

if has_tal_struct
    x = h5read([pow_dir '/' subject '_pow.hdf5'],'/tal_struct/atlases/avg.dural/x');
    y = h5read([pow_dir '/' subject '_pow.hdf5'],'/tal_struct/atlases/avg.dural/y');
    z = h5read([pow_dir '/' subject '_pow.hdf5'],'/tal_struct/atlases/avg.dural/z');
    xyz = [x y z];
elseif exist('/protocols','dir') % don't have avg.dural in power json, so look up on rhino
    
    contact_name = h5read([pow_dir '/' subject '_pow.hdf5'],'/h5info/bipolar_info/contact_name');
    montage = unique(h5read([pow_dir '/' subject '_pow.hdf5'],'/events/montage'));
    contact_name = deblank(contact_name);
    montage = deblank(montage);
    
    session = get_sessions('/protocols/r1.json', subject, 'catFR1');
    
    sess_struct = get_session_struct('/protocols/r1.json', 'r1', ...
        subject, 'catFR1', session(1));
    
    tal = get_tal_from_json(['/' sess_struct.pairs], false, 'pairs');
    xyz = vertcat(tal.xyz_avg);
    x = xyz(:,1); y = xyz(:,2); z = xyz(:,3);
else %ENS without bipolar recording
    tal = get_tal_from_json([pairs_dir '\' subject '\pairs.json'], false, [], subject);
    xyz = vertcat(tal.xyz_avg);
end

% and compute a logical array showing where contacts are shared in bipolars

if has_tal_struct
    channels = h5read([pow_dir '/' subject '_pow.hdf5'],'/tal_struct/channel')';
else
    try
        channels = h5read([pow_dir '/' subject '_pow.hdf5'],'/channel_orig');
    catch
        channels = h5read([pow_dir '/' subject '_pow.hdf5'],'/channels_orig'); %consistent naming....
        channels = [channels.ch0 channels.ch1];
    end
end

if exist('tal','var')
    if size(channels,1) ~= length(tal) % recording references exist that are not in tal struct
        new_xyz = nan(size(channels,1), size(xyz,2));
        for e = 1:size(channels,1)
            idx = find(ismember(vertcat(tal.channel), channels(e,:), 'rows'));
            if ~isempty(idx)
                new_xyz(e,:) = xyz(idx,:);
            end
        end
        xyz = new_xyz;
    end
end

bp_shared = false(size(channels,1));

for k = 1:size(bp_shared,2)
    bp_shared(k,:) = any(ismember(channels,channels(k,:)),2);
end

dist = squareform(pdist(xyz, 'Euclidean'));

try
    contact_name = h5read([pow_dir '/' subject '_pow.hdf5'],'/channels');
    montage = '0';
    contact_name = deblank(contact_name);
    contact_name = upper(contact_name);
    contact_name = strrep(contact_name,'.','');
catch
    contact_name = h5read([pow_dir '/' subject '_pow.hdf5'],'/h5info/bipolar_info/contact_name');
    montage = unique(h5read([pow_dir '/' subject '_pow.hdf5'],'/events/montage'));
    contact_name = deblank(contact_name);
    montage = deblank(montage);
end
