function eval_category_encoding(subject, pow_dir, model_dir, ...
    out_dir, rois_L, rois_R)
%EVAL_CATEGORY_ENCODING evaluates trained category model to decode encoding
% period data
%
% INPUTS
% 
% subject: string - subject identifier
% pow_dir: string - path to directory containing power hdf5s for each
%                   subject
% model_dir: string - path to directory containing trained category models
%                     for each subject 
% out_dir: string - path to directory to write output hdf5s
% rois_L: string - cell array of strings, specifying freesurfer label
%                  files in the left hemisphere (see
%                  find_network_electrode.m)
% rois_R: string - cell array of strings, specifying freesurfer label
%                  files in the right hemisphere (see
%                  find_network_electrode.m)
%
% OUTPUTS
%
% none, this function will write outputs to disk in out_dir

% By James Kragel, 2021
% Distributed under Open Source BSD License

%% prep power and event information for this subject
session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

try
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item');
catch
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item_name');
end
list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');
type = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/type');
category = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/category');

try
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'], ...
                          '/events/categoryNum');
catch
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'], ...
                           '/events/category_num');
end
% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');

% filter data/events by correct recalls that we want to analyze
type = deblank(type);

use_idx = strcmp(type,'WORD') & list > -1;
pow_all = pow_all(:,use_idx,:,:);

% update relevent variables
session = session(use_idx);
category = category(use_idx);


% zscore by session
for s = 1:length(us)
    pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

pow_all = squeeze(nanmean(pow_all(27:104,:,:,:))); % average over time

%% filter pow_all by the electrodes near the networks we are interested in
[AT, PM] = find_network_electrode(pow_dir, subject, rois_L, rois_R);

pow_PM = pow_all(:,PM,:);
pow_AT = pow_all(:,AT,:);

if sum(PM) < 5 || sum(AT) < 5 % need at least 5 electrodes in each network
    return
end

% now, test the performance of the model

test_indices = h5read([model_dir '/' subject '_model.hdf5'],'/test_indices');
model_weights =  h5read([model_dir '/' subject '_model.hdf5'],'/model_weight');
model_vecs =  h5read([model_dir '/' subject '_model.hdf5'],'/vecs');
model_item = deblank(h5read([model_dir '/' subject '_model.hdf5'],'/item'));
model_item = strrep(model_item,'ax','axe');

best_index = h5read([model_dir '/' subject '_model.hdf5'],'/best_index');
best_mse = h5read([model_dir '/' subject '_model.hdf5'],'/best_mse');
best_param = h5read([model_dir '/' subject '_model.hdf5'],'/best_param');

[X, iv] = unique(model_vecs,'rows'); % unique features the model was trained on

X_items = model_item(iv);
X_cats = lookup_cat(X_items);
category = deblank(lower(category));

pow_pred_PM = nan(size(test_indices,1),size(X,1),size(pow_PM,2),size(pow_PM,3));
pow_pred_AT = nan(size(test_indices,1),size(X,1),size(pow_AT,2),size(pow_AT,3));

for fold = 1:size(test_indices,1)
    
    % get model weights for each network, for this fold
    model_weights_PM = squeeze(model_weights(fold,:,PM,:,:));
    model_weights_AT = squeeze(model_weights(fold,:,AT,:,:));
    
    for cat = 1:size(X_cats,2)
        for e = 1:size(pow_PM,2)
            for f = 1:size(pow_PM,3)
                pow_pred_PM(fold,cat,e,f) = ...
                    squeeze(model_weights_PM(f,e,:,best_index(fold,f,e)))' * X(cat,:)';
            end
        end
        
        for e = 1:size(pow_AT,2)
            for f = 1:size(pow_AT,3)
                pow_pred_AT(fold,cat,e,f) = ...
                    squeeze(model_weights_AT(f,e,:,best_index(fold,f,e)))' * X(cat,:)';
            end
        end
    end
end


n_elec = 5;
n_it = 1000;

[u_cats, i] = unique(X_cats);
label = zeros(size(category));
for c = 1:length(category)
    label(c) = find(strcmp(category{c},u_cats));
end

PM_auc = nan(n_it, length(u_cats));
AT_auc = nan(n_it, length(u_cats));

PM_corr = nan(n_it,size(test_indices,2),size(pow_pred_PM,2));
AT_corr = nan(n_it,size(test_indices,2),size(pow_pred_AT,2));
for n = 1:n_it
    PM_idx = randperm(sum(PM)); PM_idx = PM_idx(1:n_elec);
    AT_idx = randperm(sum(AT)); AT_idx = AT_idx(1:n_elec);
    
    
    for fold = 1:size(test_indices,1)
        tmp1 = reshape(squeeze(pow_pred_PM(fold,:,PM_idx,:)),size(pow_pred_PM,2),[]);
        tmp2 = reshape(pow_PM(test_indices(fold,:)==1,PM_idx,:),sum(test_indices(fold,:)==1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        PM_corr(n,test_indices(fold,:)==1,:) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end
    
    
    for fold = 1:size(test_indices,1)
        tmp1 = reshape(squeeze(pow_pred_AT(fold,:,AT_idx,:)),size(pow_pred_AT,2),[]);
        tmp2 = reshape(pow_AT(test_indices(fold,:)==1,AT_idx,:),sum(test_indices(fold,:)==1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        AT_corr(n,test_indices(fold,:)==1,:) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end
    
    PM_corr(n,:,:) = PM_corr(n,:,i); % reorder alphabetically
    AT_corr(n,:,:) = AT_corr(n,:,i);
    
    PM_sm = softmax(squeeze(PM_corr(n,:,:))');
    AT_sm = softmax(squeeze(AT_corr(n,:,:))');
    
    for c = 1:length(u_cats)
        PM_auc(n,c) = scoreAUC(label==c,PM_sm(c,:)');
        AT_auc(n,c) = scoreAUC(label==c,AT_sm(c,:)');
    end
    
end

%write output hdf5
out_file = [out_dir '/' subject '_enc_PMAT.hdf5'];

% set up output file
if ~exist(out_file,'file')
    h5create(out_file,'/PM_corr', size(PM_corr), 'Datatype','single');
    h5create(out_file,'/AT_corr', size(AT_corr), 'Datatype','single');
    h5create(out_file,'/PM_auc', size(PM_auc), 'Datatype','single');
    h5create(out_file,'/AT_auc', size(AT_auc), 'Datatype','single');
    h5create(out_file,'/category_label', length(label));
    h5create(out_file,'/category', length(u_cats));
end

hdf5write(out_file,'/PM_corr', single(PM_corr));
hdf5write(out_file,'/AT_corr', single(AT_corr), 'WriteMode','append');
hdf5write(out_file,'/PM_auc', single(PM_auc), 'WriteMode','append');
hdf5write(out_file,'/AT_auc', single(AT_auc), 'WriteMode','append');
hdf5write(out_file,'/category_label', label, 'WriteMode','append');
hdf5write(out_file,'/category', u_cats, 'WriteMode','append');

end