function eval_category_reinstatement(subject, pow_dir, model_dir, ...
    out_dir, rois_L, rois_R)
%EVAL_CATEGORY_REINSTATEMENT evaluates trained category model to decode
% recall period data
%
% INPUTS
% 
% subject: string - subject identifier
% pow_dir: string - path to directory containing power hdf5s for each
%                   subject
% model_dir: string - path to directory containing trained category models
%                     for each subject 
% out_dir: string - path to directory to write output hdf5s
% rois_L: string - cell array of strings, specifying freesurfer label
%                  files in the left hemisphere (see
%                  find_network_electrode.m)
% rois_R: string - cell array of strings, specifying freesurfer label
%                  files in the right hemisphere (see
%                  find_network_electrode.m)
%
% OUTPUTS
%
% none, this function will write outputs to disk in out_dir

% By James Kragel, 2021
% Distributed under Open Source BSD License

%% prep power and event information for this subject

session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

try
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item');
catch
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item_name');
end
list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');
type = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/type');
% pirt = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/pirt');
% repeat = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/repeat');

rectime = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/rectime');
intrusion = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/intrusion');
category = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/category');

try
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/categoryNum');
catch
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/category_num');
end
% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');

% filter data/events by correct recalls that we want to analyze
type = deblank(type);

use_idx = strcmp(type,'REC_WORD') & intrusion == 0 & list > -1;
pow_all = pow_all(:,use_idx,:,:);

% update size of relevent variables
rectime = rectime(use_idx);
session = session(use_idx);
item = item(use_idx);
category = category(use_idx);
category_num = category_num(use_idx);
list = list(use_idx);

% compute IRT
u_lists = unique([session list],'rows');
pirt = nan(size(session));
for ul = 1:size(u_lists,1)
    idx = session == u_lists(ul,1) & list == u_lists(ul,2);
    pirt(idx) = [rectime(find(idx,1,'first')); diff(rectime(idx))];
end

% compute repeat
for ul = 1:size(u_lists)
    idx = session == u_lists(ul,1) & list == u_lists(ul,2);
    list_items = item(idx);
    repeat_tl = false(size(list_items));
    for k = 1:length(list_items)
        if any(strcmp(list_items(1:k-1),list_items{k}))
            repeat_tl(k) = true;
        end
    end
    repeat(idx) = repeat_tl;
end
repeat = repeat';
use_idx = pirt > 1500 & repeat == 0;
pow_all = pow_all(:,use_idx,:,:);

% update relevent variables
session = session(use_idx);
category = category(use_idx);


% zscore by session
for s = 1:length(us)
    pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

%% filter pow_all by the electrodes near the networks we are interested in
[AT, PM] = find_network_electrode(pow_dir, subject, rois_L, rois_R);

pow_PM = pow_all(:,:,PM,:);
pow_AT = pow_all(:,:,AT,:);

if sum(PM) < 5 || sum(AT) < 5 % need at least 5 electrodes in each network
    return
end

% now, test the performance of the model

test_indices = h5read([model_dir '/' subject '_model.hdf5'],'/test_indices');
model_weights =  h5read([model_dir '/' subject '_model.hdf5'],'/model_weight');
model_vecs =  h5read([model_dir '/' subject '_model.hdf5'],'/vecs');
model_item = deblank(h5read([model_dir '/' subject '_model.hdf5'],'/item'));
model_item = strrep(model_item,'ax','axe');

best_index = h5read([model_dir '/' subject '_model.hdf5'],'/best_index');
best_mse = h5read([model_dir '/' subject '_model.hdf5'],'/best_mse');
best_param = h5read([model_dir '/' subject '_model.hdf5'],'/best_param');

[X, iv] = unique(model_vecs,'rows'); % unique features the model was trained on

X_items = model_item(iv);
X_cats = lookup_cat(X_items);
category = deblank(lower(category));

pred_activity_PM = nan(5,8,size(pow_PM,3),size(X,1));
pred_activity_AT = nan(5,8,size(pow_AT,3),size(X,1));
for fold = 1:5
    
    % get model weights for each network, for this fold
    model_weights_PM = squeeze(model_weights(fold,:,PM,:,:));
    model_weights_AT = squeeze(model_weights(fold,:,AT,:,:));   
     
    for cat = 1:size(X_cats,2)
        for e = 1:size(pow_PM,3)
            for f = 1:size(pow_PM,4)
                pred_activity_PM(fold,f,e,cat) = ...
                    squeeze(model_weights_PM(f,e,:,best_index(fold,f,e)))' * X(cat,:)';
            end
        end
        
        for e = 1:size(pow_AT,3)
            for f = 1:size(pow_AT,4)
                pred_activity_AT(fold,f,e,cat) = ...
                    squeeze(model_weights_AT(f,e,:,best_index(fold,f,e)))' * X(cat,:)';
            end
        end
    end
    
end

pred_activity_PM = squeeze(nanmean(pred_activity_PM));
pred_activity_PM = permute(pred_activity_PM,[3 2 1]);

pred_activity_AT = squeeze(nanmean(pred_activity_AT));
pred_activity_AT = permute(pred_activity_AT,[3 2 1]);

% correlate predicted activity to observed activity across recalls

pow_PM = permute(pow_PM,[2 1 3 4]);
pow_AT = permute(pow_AT,[2 1 3 4]);

pow_PM = permute(pow_PM, [1 3 4 2]);
pow_AT = permute(pow_AT, [1 3 4 2]);

n_elec = 5;
n_it = 1000;

[u_cats, i] = unique(X_cats);
label = zeros(size(category));
for c = 1:length(category)
    label(c) = find(strcmp(category{c},u_cats));
end


PM_auc = nan(n_it, length(u_cats), size(pow_PM,4));
AT_auc = nan(n_it, length(u_cats), size(pow_AT,4));

PM_corr = nan(n_it,size(pow_PM,1),size(pred_activity_PM,1),size(pow_PM,4));
AT_corr = nan(n_it,size(pow_AT,1),size(pred_activity_AT,1),size(pow_AT,4));
for n = 1:n_it
    
    PM_idx = randperm(sum(PM)); PM_idx = PM_idx(1:n_elec);
    AT_idx = randperm(sum(AT)); AT_idx = AT_idx(1:n_elec);
    
    %
    for t = 1:size(pow_PM,4)
        tmp1 = reshape(squeeze(pred_activity_PM(:,PM_idx,:)),size(pred_activity_PM,1),[]);
        tmp2 = reshape(pow_PM(:,PM_idx,:,t),size(pow_PM,1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        PM_corr(n,:,:,t) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end
    
    for t = 1:size(pow_AT,4)
        tmp1 = reshape(squeeze(pred_activity_AT(:,AT_idx,:)),size(pred_activity_AT,1),[]);
        tmp2 = reshape(pow_AT(:,AT_idx,:,t),size(pow_AT,1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        AT_corr(n,:,:,t) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end

    PM_corr(n,:,:,:) = PM_corr(n,:,i,:);
    AT_corr(n,:,:,:) = AT_corr(n,:,i,:);
    
    % TODO: init sms
    for t = 1:40
        PM_sm(t,:,:) = softmax(squeeze(PM_corr(n,:,:,t))');
        AT_sm(t,:,:) = softmax(squeeze(AT_corr(n,:,:,t))');
    end
    
    for c = 1:length(u_cats)
        for t = 1:40
            if any(label==c)
                PM_auc(n,c,t) = scoreAUC(label==c, squeeze(PM_sm(t,c,:)));
                AT_auc(n,c,t) = scoreAUC(label==c,squeeze(AT_sm(t,c,:)));
            end
        end
    end
end

%write output hdf5
out_file = [out_dir '/' subject '_reins_PMAT.hdf5'];

% set up output file
if ~exist(out_file,'file')
    h5create(out_file,'/PM_sm', size(PM_sm), 'Datatype','single');
    h5create(out_file,'/AT_sm', size(AT_sm), 'Datatype','single');
    h5create(out_file,'/PM_auc', size(PM_auc), 'Datatype','single');
    h5create(out_file,'/AT_auc', size(AT_auc), 'Datatype','single');
    h5create(out_file,'/category_label', length(label));
    h5create(out_file,'/category', length(u_cats));
end

hdf5write(out_file,'/PM_sm', single(PM_sm));
hdf5write(out_file,'/AT_sm', single(AT_sm), 'WriteMode','append');
hdf5write(out_file,'/PM_auc', single(PM_auc), 'WriteMode','append');
hdf5write(out_file,'/AT_auc', single(AT_auc), 'WriteMode','append');
hdf5write(out_file,'/category_label', label, 'WriteMode','append');
hdf5write(out_file,'/category', u_cats, 'WriteMode','append');


end