function [params, goodness, dist, c_all] = eval_conn_by_dist(subject, ...
    enc_pow_dir, rec_pow_dir, pairs_dir)
%EVAL_CONN_BY_DIST runs analysis to examine connectivity by distance
%
%
% INPUTS
%
% subject: string - subject identifier
% enc_pow_dir: string - path to directory containing encoding
%                       power hdf5s for each subject
% rec_pow_dir: string - path to directory containing recall
%                       power hdf5s for each subject
% pairs_dir: string - directory pointing to pairs.json for data recorded on
%                     the ens system
%
%
%
% OUTPUTS
%
% params: struct - contains parameters for exponential decay model
%                  describing relation between connectivity and distance
% goodness: stucture - contains fit measures for the model
% dist: double - distance between electrodes
% c_all: double - c parameter from model fitting

% By James Kragel, 2021
% Distributed under Open Source BSD License

pow_enc = get_enc_power(enc_pow_dir, subject);
pow_rec = get_rec_power(rec_pow_dir, subject);

[dist, bp_shared] = compute_elec_dists(enc_pow_dir, subject, pairs_dir);

dist(bp_shared) = nan;
ref = eye(size(dist));

dist(ref==1)=0;
dist = squareform(dist);

pow = vertcat(pow_enc, pow_rec);

% fitting options

pctRunOnAll warning off

c_all = nan(size(pow,3), length(dist));
for i = 1:size(pow,3)

    c =  min(dist(dist~=0));
    g = fittype( @(a,b,x) a*exp(-((x-c)*b) ) );
    f = fitoptions('Method','NonlinearLeastSquare');
    f.Robust = 'LAR';
    f.Display = 'Off';
    f.Lower = [0 0];
    f.Upper = [1 10];

    c = corr(squeeze(pow(:,:,i)));
    c(ref==1)=0;
    c_all(i,:) = squareform(c);
    
    % perform nonlinear regression using robust options
    g_idx = ~isnan(dist) & ~isnan(c_all(i,:));
    [curve, goodness(i)] = fit(dist(g_idx)', c_all(i,g_idx)', g, f);
    
    params.a(i) = curve.a;
    params.b(i) = curve.b;
    
    % fit null models by permuting dist
    null_a = nan(1,100);
    null_b = nan(1,100);
    parfor n = 1:100
        c = min(dist(dist~=0));
        g = fittype( @(a,b,x) a*exp(-((x-c)*b) ) );
        f = fitoptions('Method','NonlinearLeastSquare');
        f.Robust = 'LAR';
        f.Display = 'Off';
        f.Lower = [0 0];
        f.Upper = [1 10];
        
        perm_idx = find(g_idx); perm_idx = perm_idx(randperm(length(perm_idx)));
        curve = fit(dist(perm_idx)', c_all(i,g_idx)', g, f);
        
        null_a(n) = curve.a;
        null_b(n) = curve.b;
    end
    
    % convert params to zscore from null for group inference
    params.z_a(i) = (params.a(i)-mean(null_a))./std(null_a);
    params.z_b(i) = (params.b(i)-mean(null_b))./std(null_b);
    
end

end

function pow_all = get_enc_power(pow_dir, subject)

%% prep power and event information for this subject
session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');
type = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/type');

% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');

% filter data/events by correct recalls that we want to analyze
type = deblank(type);

use_idx = strcmp(type,'WORD') & list > -1;
pow_all = pow_all(:,use_idx,:,:);

% update size of relevent variables
session = session(use_idx);

% zscore by session
for s = 1:length(us)
    pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

pow_all = squeeze(nanmean(pow_all(27:104,:,:,:))); % average over time


end

function pow_all = get_rec_power(pow_dir, subject)


session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

try
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item');
catch
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item_name');
end
list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');
type = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/type');

rectime = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/rectime');
intrusion = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/intrusion');
% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');

% filter data/events by correct recalls that we want to analyze
type = deblank(type);

use_idx = strcmp(type,'REC_WORD') & intrusion == 0 & list > -1;
pow_all = pow_all(:,use_idx,:,:);

% update size of relevent variables
rectime = rectime(use_idx);
session = session(use_idx);
item = item(use_idx);
list = list(use_idx);

% compute IRT
u_lists = unique([session list],'rows');
pirt = nan(size(session));
for ul = 1:size(u_lists,1)
    idx = session == u_lists(ul,1) & list == u_lists(ul,2);
    pirt(idx) = [rectime(find(idx,1,'first')); diff(rectime(idx))];
end

% compute repeat
for ul = 1:size(u_lists)
    idx = session == u_lists(ul,1) & list == u_lists(ul,2);
    list_items = item(idx);
    repeat_tl = false(size(list_items));
    for k = 1:length(list_items)
        if any(strcmp(list_items(1:k-1),list_items{k}))
            repeat_tl(k) = true;
        end
    end
    repeat(idx) = repeat_tl;
end
repeat = repeat';
use_idx = pirt > 1500 & repeat == 0;
pow_all = pow_all(:,use_idx,:,:);

session = session(use_idx);

% zscore by session
for s = 1:length(us)
    pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

pow_all = squeeze(nanmean(pow_all));

end
