function [n_elec_tab, soz_tab] = eval_decoding()
% EVAL_DECODING runs stats for decoding performance by number of
% electrodes, and stats for connectivity as a function of difference
%
% INPUTS
%
%  this function does not have any inputs, but rather loads intermediate
%  results files
%
% OUTPUTS
%
% n_elec_tab: likelihood ratio and p-value stats for examining the effect
% of number of electrodes on decoding performance
%
% soz_tab: likelihood ratio and p-value stats for examining the effect
% of electrodes in soz (or otherwise showing epileptogenic activity) on
% decoding performance


% By James Kragel, 2021
% Distributed under Open Source BSD License

% run anova for auc_by_elec (n feature)

[tab_PM, tab_AT] = get_tables(false); % all
[tab_PM_soz, tab_AT_soz] = get_tables(true); % exclude soz

% encoding
[res_enc_pos_PM, ~, ~] =  fit_compare_restricted(tab_PM, 'enc_pos');
[res_enc_cat_PM, ~, ~] =  fit_compare_restricted(tab_PM, 'enc_cat');
[res_enc_pos_AT, ~, ~] =  fit_compare_restricted(tab_AT, 'enc_pos');
[res_enc_cat_AT, ~, ~] =  fit_compare_restricted(tab_AT, 'enc_cat');

% reinstatement
[res_rec_pos_PM, ~, ~] =  fit_compare_restricted(tab_PM, 'rec_pos');
[res_rec_cat_PM, ~, ~] =  fit_compare_restricted(tab_PM, 'rec_cat');
[res_rec_pos_AT, ~, ~] =  fit_compare_restricted(tab_AT, 'rec_pos');
[res_rec_cat_AT, ~, ~] =  fit_compare_restricted(tab_AT, 'rec_cat');

n_elec_tab(1,:) = [res_enc_pos_PM.LRStat(2) res_enc_pos_PM.pValue(2)];
n_elec_tab(2,:) = [res_enc_cat_PM.LRStat(2) res_enc_cat_PM.pValue(2)];
n_elec_tab(3,:) = [res_rec_pos_PM.LRStat(2) res_rec_pos_PM.pValue(2)];
n_elec_tab(4,:) = [res_rec_cat_PM.LRStat(2) res_rec_cat_PM.pValue(2)];

n_elec_tab(5,:) = [res_enc_pos_AT.LRStat(2) res_enc_pos_AT.pValue(2)];
n_elec_tab(6,:) = [res_enc_cat_AT.LRStat(2) res_enc_cat_AT.pValue(2)];
n_elec_tab(7,:) = [res_rec_pos_AT.LRStat(2) res_rec_pos_AT.pValue(2)];
n_elec_tab(8,:) = [res_rec_cat_AT.LRStat(2) res_rec_cat_AT.pValue(2)];


% compare effect of excluding soz
res_enc_pos_PM_soz = fit_compare_soz(tab_PM, tab_PM_soz, 'enc_pos');
res_enc_cat_PM_soz = fit_compare_soz(tab_PM, tab_PM_soz, 'enc_cat');
res_rec_pos_PM_soz = fit_compare_soz(tab_PM, tab_PM_soz, 'rec_pos');
res_rec_cat_PM_soz = fit_compare_soz(tab_PM, tab_PM_soz, 'rec_cat');

res_enc_pos_AT_soz = fit_compare_soz(tab_AT, tab_AT_soz, 'enc_pos');
res_enc_cat_AT_soz = fit_compare_soz(tab_AT, tab_AT_soz, 'enc_cat');
res_rec_pos_AT_soz = fit_compare_soz(tab_AT, tab_AT_soz, 'rec_pos');
res_rec_cat_AT_soz = fit_compare_soz(tab_AT, tab_AT_soz, 'rec_cat');

soz_tab(1,:) = [res_enc_pos_PM_soz.LRStat(2) res_enc_pos_PM_soz.pValue(2)];
soz_tab(2,:) = [res_enc_cat_PM_soz.LRStat(2) res_enc_cat_PM_soz.pValue(2)];
soz_tab(3,:) = [res_rec_pos_PM_soz.LRStat(2) res_rec_pos_PM_soz.pValue(2)];
soz_tab(4,:) = [res_rec_cat_PM_soz.LRStat(2) res_rec_cat_PM_soz.pValue(2)];

soz_tab(5,:) = [res_enc_pos_AT_soz.LRStat(2) res_enc_pos_AT_soz.pValue(2)];
soz_tab(6,:) = [res_enc_cat_AT_soz.LRStat(2) res_enc_cat_AT_soz.pValue(2)];
soz_tab(7,:) = [res_rec_pos_AT_soz.LRStat(2) res_rec_pos_AT_soz.pValue(2)];
soz_tab(8,:) = [res_rec_cat_AT_soz.LRStat(2) res_rec_cat_AT_soz.pValue(2)];

end

function res = fit_compare_soz(tab, tab_exclude, yvar)

tab.exclude = false(size(tab,1),1);
tab_exclude.exclude = true(size(tab_exclude,1),1);
tab_all = cat(1, tab, tab_exclude);

glme = fitglme(tab_all,...
    [yvar ' ~ 1 + n_elec + exclude + (1+exclude+n_elec|subj)'],...
    'Distribution', 'Normal', ...
    'FitMethod', 'Laplace');

glme_r = fitglme(tab_all,...
    [yvar ' ~ 1 + n_elec + (1+exclude+n_elec|subj)'],...
    'Distribution', 'Normal', ...
    'FitMethod', 'Laplace');

res = compare(glme_r, glme);


end

function [res, glme, glme_r] =  fit_compare_restricted(tab, yvar)

glme = fitglme(tab,...
    [yvar ' ~ 1 + n_elec + (1+n_elec|subj)'],...
    'Distribution', 'Normal', ...
    'FitMethod', 'Laplace');

glme_r = fitglme(tab,...
    [yvar ' ~ 1 + (1+n_elec|subj)'],...
    'Distribution', 'Normal', ...
    'FitMethod', 'Laplace');

res = compare(glme_r, glme);

end

function [tab_PM, tab_AT] = get_tables(exclude)

if ~exclude
    res = auc_by_elec(false);
%     load('res_all_elec.mat', 'res');
else
    res = auc_by_elec(true);
%     load('res_soz_exclude.mat', 'res');
end
    
% network and n_elec
x_n_elec = repmat(1:50,69,1);
x_subj = repmat((1:69)',1,50);

y_enc_pos = res.PM_pos';
y_enc_cat = res.PM_cat';
y_rec_pos = squeeze(nanmean(res.PM_pos_reins,2))';
y_rec_cat = squeeze(nanmean(res.PM_cat_reins,2))';

tab_PM = table(y_enc_pos(:), ...
    y_enc_cat(:), ...
    y_rec_pos(:), ...
    y_rec_cat(:), ...
    x_n_elec(:), ...
    categorical(x_subj(:)), ...
    'VariableNames', ...
    {'enc_pos', 'enc_cat', ...
    'rec_pos', 'rec_cat', ...
    'n_elec', 'subj'});

y_enc_pos = res.AT_pos';
y_enc_cat = res.AT_cat';
y_rec_pos = squeeze(nanmean(res.AT_pos_reins,2))';
y_rec_cat = squeeze(nanmean(res.AT_cat_reins,2))';

tab_AT = table(y_enc_pos(:), ...
    y_enc_cat(:), ...
    y_rec_pos(:), ...
    y_rec_cat(:), ...
    x_n_elec(:), ...
    categorical(x_subj(:)), ...
    'VariableNames', ...
    {'enc_pos', 'enc_cat', ...
    'rec_pos', 'rec_cat', ...
    'n_elec', 'subj'});

end

function res = auc_by_elec(soz_flag)
%AUC_BY_ELEC load and organize results

% encoding
pos_dir = '..\..\data\encoding_serialpos_by_elec\';
if soz_flag
    pos_dir = [pos_dir '_soz'];
end
[res.PM_pos, res.AT_pos] = load_aucs(pos_dir, 'auc_spos');

cat_dir = '..\..\data\encoding_category_by_elec\';
if soz_flag
    cat_dir = [cat_dir '_soz'];
end
[res.PM_cat, res.AT_cat] = load_aucs(cat_dir, 'auc');

% retrieval
pos_reins_dir = '..\..\data\retrieval_serialpos_by_elec\';
if soz_flag
    pos_reins_dir = [pos_reins_dir '_soz'];
end
[res.PM_pos_reins, res.AT_pos_reins] = load_reins(pos_reins_dir, 'auc_spos');

cat_reins_dir = '..\..\data\retrieval_category_by_elec\';
if soz_flag
    cat_reins_dir = [cat_reins_dir '_soz'];
end
[res.PM_cat_reins, res.AT_cat_reins] = load_reins(cat_reins_dir, 'auc');


% and plot the results

% plot encoding

figure;
subplot(2,1,1);

x = [1:10; 1:10];
y = [nanmean(res.PM_pos(1:10,:),2) nanmean(res.AT_pos(1:10,:),2)];
e = [nansem(res.PM_pos(1:10,:),[],2) nansem(res.AT_pos(1:10,:),[],2)];

map = lines(5);
map = map(4:5,:);

[hl, hp] = boundedline(x', ...
    y, ...
    e, ...
    'cmap',map);

hl(1).LineWidth = 2;
hl(2).LineWidth = 2;

title('Serial Position');
set(gca,'LineWidth', 2, 'FontSize',14, 'XLim', [1 10]);
ylabel('Decoding Accuracy (AUC)');
xlabel('Number of Electrodes');

subplot(2,1,2);

y = [nanmean(res.PM_cat(1:10,:),2) nanmean(res.AT_cat(1:10,:),2)];
e = [nansem(res.PM_cat(1:10,:),[],2) nansem(res.AT_cat(1:10,:),[],2)];

[hl, hp] = boundedline(x', ...
    y, ...
    e, ...
    'cmap',map);

hl(1).LineWidth = 2;
hl(2).LineWidth = 2;

title('Category Decoding');
set(gca,'LineWidth', 2, 'FontSize',14, 'XLim', [1 10]);
ylabel('Decoding Accuracy (AUC)');
xlabel('Number of Electrodes');

set(gcf,'Position', [962   291   330   624]);

% temporal reinstatement
figure;
ax1 = subplot(1,2,1);
imagesc(squeeze(nanmean(res.PM_pos_reins,3)));
set(gca,'CLim',[.5 .53], 'YLim',[0.5 10.5]);
title('Serial Position');
colormap(ax1, purples);
hc = colorbar;
set(ax1,'XTick', [1 20 40], ...
    'YTick', [1 5 10], ...
    'XTickLabel', [-800 -400 0], ...
    'FontSize',12, ...
    'YDir', 'normal');

ylabel('Number of Electrodes')
xlabel('Time (ms)')

ax2 = subplot(1,2,2);
imagesc(squeeze(nanmean(res.AT_pos_reins,3)));
set(gca,'CLim',[.5 .53], 'YLim',[0.5 10.5]);
title('Serial Position');
colormap(ax2, greens);
hc = colorbar;
set(ax2,'XTick', [1 20 40], ...
    'YTick', [1 5 10], ...
    'XTickLabel', [-800 -400 0], ...
    'FontSize',12, ...
    'YDir', 'normal')

ylabel('Number of Electrodes')
xlabel('Time (ms)')
set(gcf,'Position', [796   392   631   250]);

% category reinstatement
figure;
ax1 = subplot(1,2,1);
imagesc(squeeze(nanmean(res.PM_cat_reins,3)));
set(gca,'CLim',[.5 .55], 'YLim',[0.5 10.5]);
title('Category');
colormap(ax1, purples);
hc = colorbar;
set(ax1,'XTick', [1 20 40], ...
    'YTick', [1 5 10], ...
    'XTickLabel', [-800 -400 0], ...
    'FontSize',12, ...
    'YDir', 'normal');

ylabel('Number of Electrodes')
xlabel('Time (ms)')

ax2 = subplot(1,2,2);
imagesc(squeeze(nanmean(res.AT_cat_reins,3)));
set(gca,'CLim',[.5 .55], 'YLim',[0.5 10.5]);
title('Category');
colormap(ax2, greens);
hc = colorbar;
set(ax2,'XTick', [1 20 40], ...
    'YTick', [1 5 10], ...
    'XTickLabel', [-800 -400 0], ...
    'FontSize',12, ...
    'YDir', 'normal')

ylabel('Number of Electrodes')
xlabel('Time (ms)')
set(gcf,'Position', [796   392   631   250]);

end

function [PM_auc, AT_auc] = load_aucs(res_dir, fieldname)

cd(res_dir);
a = dir('*.hdf5');
PM_auc = nan(50,69);
AT_auc = nan(50,69);
for k = 1:length(a)
    try
    tmp = squeeze(nanmean(h5read(a(k).name, ['/PM_' fieldname]),3));
    PM_auc(1:size(tmp,1),k) =  nanmean(tmp,2);
    tmp = squeeze(nanmean(h5read(a(k).name, ['/AT_' fieldname]),3));
    AT_auc(1:size(tmp,1),k) = nanmean(tmp,2);
    end
end

% below is for subjects with insufficient n_elec
PM_auc(PM_auc==0)=nan;
AT_auc(AT_auc==0)=nan;

end

function [PM_auc, AT_auc] = load_reins(res_dir, fieldname)

cd(res_dir);
a = dir('*.hdf5');
PM_auc = nan(50,40,69);
AT_auc = nan(50,40,69);
for k = 1:length(a)
    try
        tmp = squeeze(nanmean(nanmean(h5read(a(k).name, ['/PM_' fieldname]),2),3));
        PM_auc(1:size(tmp,1),:,k) = tmp;
        tmp = squeeze(nanmean(nanmean(h5read(a(k).name, ['/AT_' fieldname]),2),3));
        AT_auc(1:size(tmp,1),:,k) = tmp;
    end
end

PM_auc(PM_auc==0)=nan;
AT_auc(AT_auc==0)=nan;

end