function data = eval_network_conn(subject, enc_pow_dir, rec_pow_dir, rois_L, rois_R)
%EVAL_CONN_BY_DIST runs analysis to examine connectivity by distance
%
%
% INPUTS
%
% subject: string - subject identifier
% enc_pow_dir: string - path to directory containing encoding
%                       power hdf5s for each subject
% rec_pow_dir: string - path to directory containing recall
%                       power hdf5s for each subject
% rois_L: string - cell array of strings, specifying freesurfer label
%                  files in the left hemisphere (see
%                  find_network_electrode.m)
% rois_R: string - cell array of strings, specifying freesurfer label
%                  files in the right hemisphere (see
%                  find_network_electrode.m)
%
% OUTPUTS
%
% params: struct - contains parameters for exponential decay model
%                  describing relation between connectivity and distance
% goodness: stucture - contains fit measures for the model
% dist: double - distance between electrodes
% c_all: double - c parameter from model fitting

% By James Kragel, 2021
% Distributed under Open Source BSD License


pow_enc = get_enc_power(enc_pow_dir, subject);
pow_rec = get_rec_power(rec_pow_dir, subject);

[AT, PM] = find_network_electrode(enc_pow_dir, subject, rois_L, rois_R);

[dist, bp_shared] = elec_dists(enc_pow_dir, subject);

dist(bp_shared) = nan;

pow = vertcat(pow_enc, pow_rec);

[owAT_idx, owPM_idx, obetween_idx] = matched_randsample(AT, PM, dist, bp_shared);

data  = nan(size(pow,3),1000,3);
for f = 1:size(pow,3)
    c = corr(squeeze(pow(:,:,f)));
    c = tril(c,-1); c(c==0)=nan;
    c(bp_shared) = nan;
    
    c = double(atanh(c));  
    
    for n = 1:size(owAT_idx,1)
        atidx = owAT_idx(n,:); atidx(isnan(atidx))=[];
        pmidx = owPM_idx(n,:); pmidx(isnan(pmidx))=[];
        bidx = obetween_idx(n,:); bidx(isnan(bidx))=[];
        
        data(f,n,:) = [nanmean(c(atidx),2), nanmean(c(pmidx),2), nanmean(c(bidx),2)];
    end
    
end

end

function pow_all = get_enc_power(pow_dir, subject)

%% prep power and event information for this subject
session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');
type = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/type');

% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');

% filter data/events by correct recalls that we want to analyze
type = deblank(type);

use_idx = strcmp(type,'WORD') & list > -1;
pow_all = pow_all(:,use_idx,:,:);

% update relevent variables
session = session(use_idx);

% zscore by session
for s = 1:length(us)
    pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

pow_all = squeeze(nanmean(pow_all(27:104,:,:,:))); % average over time


end

function pow_all = get_rec_power(pow_dir, subject)

session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

try
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item');
catch
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item_name');
end
list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');
type = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/type');
% pirt = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/pirt');
% repeat = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/repeat');

rectime = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/rectime');
intrusion = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/intrusion');

% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');

% filter data/events by correct recalls that we want to analyze
type = deblank(type);

use_idx = strcmp(type,'REC_WORD') & intrusion == 0 & list > -1;
pow_all = pow_all(:,use_idx,:,:);

% update size of relevent variables
rectime = rectime(use_idx);
session = session(use_idx);
item = item(use_idx);
list = list(use_idx);

% compute IRT
u_lists = unique([session list],'rows');
pirt = nan(size(session));
for ul = 1:size(u_lists,1)
    idx = session == u_lists(ul,1) & list == u_lists(ul,2);
    pirt(idx) = [rectime(find(idx,1,'first')); diff(rectime(idx))];
end

% compute repeat
for ul = 1:size(u_lists)
    idx = session == u_lists(ul,1) & list == u_lists(ul,2);
    list_items = item(idx);
    repeat_tl = false(size(list_items));
    for k = 1:length(list_items)
        if any(strcmp(list_items(1:k-1),list_items{k}))
            repeat_tl(k) = true;
        end
    end
    repeat(idx) = repeat_tl;
end
repeat = repeat';
use_idx = pirt > 1500 & repeat == 0;
pow_all = pow_all(:,use_idx,:,:);

% update relevent variables
session = session(use_idx);


% zscore by session
for s = 1:length(us)
    pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

pow_all = squeeze(nanmean(pow_all)); % average over time (16:end,:,:,:)

end
