function eval_serialpos_encoding(subject, pow_dir, model_dir, out_dir, rois_L, rois_R)
%EVAL_SERIALPOS_ENCODING evaluates trained serial position model to decode
% encoding period data
%
% INPUTS
% 
% subject: string - subject identifier
% pow_dir: string - path to directory containing power hdf5s for each
%                   subject
% model_dir: string - path to directory containing trained category models
%                     for each subject 
% out_dir: string - path to directory to write output hdf5s
% rois_L: string - cell array of strings, specifying freesurfer label
%                  files in the left hemisphere (see
%                  find_network_electrode.m)
% rois_R: string - cell array of strings, specifying freesurfer label
%                  files in the right hemisphere (see
%                  find_network_electrode.m)
%
% OUTPUTS
%
% none, this function will write outputs to disk in out_dir

% By James Kragel, 2021
% Distributed under Open Source BSD License

%% prep power and event information for this subject
session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

try
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item');
catch
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item_name');
end
list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');
serialpos = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/serialpos');
type = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/type');
category = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/category');

try
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/categoryNum');
catch
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/category_num');
end
% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');

% filter data/events by correct recalls that we want to analyze
type = deblank(type);

use_idx = strcmp(type,'WORD') & list > -1;
pow_all = pow_all(:,use_idx,:,:);

% update relevent variables
session = session(use_idx);
list = list(use_idx);
serialpos = serialpos(use_idx);

% zscore by session
for s = 1:length(us)
   pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

pow_all = squeeze(nanmean(pow_all(27:104,:,:,:))); % average over time

%% filter pow_all by the electrodes near the networks we are interested in
[AT, PM] = find_network_electrode(pow_dir, subject, rois_L, rois_R);

pow_PM = pow_all(:,PM,:);
pow_AT = pow_all(:,AT,:);

if sum(PM) < 5 || sum(AT) < 5 % need at least 5 electrodes in each network
    return
end

% now, test the performance of the model

test_indices = h5read([model_dir '/' subject '_model.hdf5'],'/test_indices');
model_weights =  h5read([model_dir '/' subject '_model.hdf5'],'/model_weight');
model_vecs =  h5read([model_dir '/' subject '_model.hdf5'],'/vecs');

best_index = h5read([model_dir '/' subject '_model.hdf5'],'/best_index');
best_mse = h5read([model_dir '/' subject '_model.hdf5'],'/best_mse');
best_param = h5read([model_dir '/' subject '_model.hdf5'],'/best_param');

spos_idx = 1:length(unique(serialpos));
list_idx = spos_idx(end)+1:spos_idx(end)+length(unique(list));
sess_idx = list_idx(end)+1:list_idx(end)+length(unique(session));

X = model_vecs;

pow_pred_PM = nan(size(pow_PM,1),size(X,1),size(pow_PM,2),size(pow_PM,3));
pow_pred_AT = nan(size(pow_AT,1),size(X,1),size(pow_AT,2),size(pow_AT,3));

for fold = 1:size(test_indices,1)

    % get model weights for each network, for this fold
    model_weights_PM = squeeze(model_weights(fold,:,PM,:,:));
    model_weights_AT = squeeze(model_weights(fold,:,AT,:,:));
    
    for val = 1:size(X,1)
        for e = 1:size(pow_PM,2)
            for f = 1:size(pow_PM,3)
                pow_pred_PM(test_indices(fold,:)==1,val,e,f) = ...
                    squeeze(model_weights_PM(f,e,:,best_index(fold,f,e)))' * X(val,:)';
            end
        end
        
        for e = 1:size(pow_AT,2)
            for f = 1:size(pow_AT,3)
                pow_pred_AT(test_indices(fold,:)==1,val,e,f) = ...
                    squeeze(model_weights_AT(f,e,:,best_index(fold,f,e)))' * X(val,:)';
            end
        end
    end
end

% average over predicted activity to create marginal means 

% spos
pow_pred_PM_spos = nan(length(spos_idx),size(pow_pred_PM,3),size(pow_pred_PM,4));
for s = 1:length(spos_idx)
     pow_pred_PM_spos(s,:,:) = squeeze(nanmean(nanmean(pow_pred_PM(:,model_vecs(:,spos_idx(s))==1,:,:))));
end

pow_pred_AT_spos = nan(length(spos_idx),size(pow_pred_AT,3),size(pow_pred_AT,4));
for s = 1:length(spos_idx)
     pow_pred_AT_spos(s,:,:) = squeeze(nanmean(nanmean(pow_pred_AT(:,model_vecs(:,spos_idx(s))==1,:,:))));
end

% list
pow_pred_PM_list = nan(length(list_idx),size(pow_pred_PM,3),size(pow_pred_PM,4));
for s = 1:length(list_idx)
     pow_pred_PM_list(s,:,:) = squeeze(nanmean(nanmean(pow_pred_PM(:,model_vecs(:,list_idx(s))==1,:,:))));
end

pow_pred_AT_list = nan(length(list_idx),size(pow_pred_AT,3),size(pow_pred_AT,4));
for s = 1:length(list_idx)
     pow_pred_AT_list(s,:,:) = squeeze(nanmean(nanmean(pow_pred_AT(:,model_vecs(:,list_idx(s))==1,:,:))));
end

% sess
pow_pred_PM_sess = nan(length(sess_idx),size(pow_pred_PM,3),size(pow_pred_PM,4));
for s = 1:length(sess_idx)
     pow_pred_PM_sess(s,:,:) = squeeze(nanmean(nanmean(pow_pred_PM(:,model_vecs(:,sess_idx(s))==1,:,:))));
end

pow_pred_AT_sess = nan(length(sess_idx),size(pow_pred_AT,3),size(pow_pred_AT,4));
for s = 1:length(sess_idx)
     pow_pred_AT_sess(s,:,:) = squeeze(nanmean(nanmean(pow_pred_AT(:,model_vecs(:,sess_idx(s))==1,:,:))));
end

% perform 1000 iterations, randomly sampling the same number of electrodes
% from each network

n_elec = 5; %min(sum(PM),sum(AT));
n_it = 1000;

[~,~,spos_label] = unique(serialpos);
[~,~,list_label] = unique(list);
[~,~,sess_label] = unique(session);

PM_auc_spos = nan(n_it, length(unique(serialpos)));
AT_auc_spos = nan(n_it, length(unique(serialpos)));

PM_auc_list = nan(n_it, length(unique(list)));
AT_auc_list = nan(n_it, length(unique(list)));

PM_auc_sess = nan(n_it, length(unique(session)));
AT_auc_sess = nan(n_it, length(unique(session)));

for n = 1:n_it
    PM_idx = randperm(sum(PM)); PM_idx = PM_idx(1:n_elec);
    AT_idx = randperm(sum(AT)); AT_idx = AT_idx(1:n_elec);
          
    % spos
    tmp1 = reshape(pow_pred_PM_spos(:,PM_idx,:),size(pow_pred_PM_spos,1),[]);
    tmp2 = reshape(pow_PM(:,PM_idx,:),size(pow_PM,1),[]);
    PM_corr_spos = corr(vertcat(tmp1,tmp2)','rows','complete');
    PM_corr_spos = PM_corr_spos(1:size(tmp1,1),size(tmp1,1)+1:end)';
    
    tmp1 = reshape(pow_pred_AT_spos(:,AT_idx,:),size(pow_pred_AT_spos,1),[]);
    tmp2 = reshape(pow_AT(:,AT_idx,:),size(pow_AT,1),[]);
    AT_corr_spos = corr(vertcat(tmp1,tmp2)','rows','complete');
    AT_corr_spos = AT_corr_spos(1:size(tmp1,1),size(tmp1,1)+1:end)';
    
    % list
    tmp1 = reshape(pow_pred_PM_list(:,PM_idx,:),size(pow_pred_PM_list,1),[]);
    tmp2 = reshape(pow_PM(:,PM_idx,:),size(pow_PM,1),[]);
    PM_corr_list = corr(vertcat(tmp1,tmp2)','rows','complete');
    PM_corr_list = PM_corr_list(1:size(tmp1,1),size(tmp1,1)+1:end)';
    
    tmp1 = reshape(pow_pred_AT_list(:,AT_idx,:),size(pow_pred_AT_list,1),[]);
    tmp2 = reshape(pow_AT(:,AT_idx,:),size(pow_AT,1),[]);
    AT_corr_list = corr(vertcat(tmp1,tmp2)','rows','complete');
    AT_corr_list = AT_corr_list(1:size(tmp1,1),size(tmp1,1)+1:end)';
    
    % sess
    tmp1 = reshape(pow_pred_PM_sess(:,PM_idx,:),size(pow_pred_PM_sess,1),[]);
    tmp2 = reshape(pow_PM(:,PM_idx,:),size(pow_PM,1),[]);
    PM_corr_sess = corr(vertcat(tmp1,tmp2)','rows','complete');
    PM_corr_sess = PM_corr_sess(1:size(tmp1,1),size(tmp1,1)+1:end)';
    
    tmp1 = reshape(pow_pred_AT_sess(:,AT_idx,:),size(pow_pred_AT_sess,1),[]);
    tmp2 = reshape(pow_AT(:,AT_idx,:),size(pow_AT,1),[]);
    AT_corr_sess = corr(vertcat(tmp1,tmp2)','rows','complete');
    AT_corr_sess = AT_corr_sess(1:size(tmp1,1),size(tmp1,1)+1:end)';

    % and compute AUC
    PM_sm_spos = softmax(PM_corr_spos');
    AT_sm_spos = softmax(AT_corr_spos');
    
    PM_sm_list = softmax(PM_corr_list');
    AT_sm_list = softmax(AT_corr_list');
    
    PM_sm_sess = softmax(PM_corr_sess');
    AT_sm_sess = softmax(AT_corr_sess');
    
    for c = 1:size(PM_sm_spos,1)
        PM_auc_spos(n,c) = scoreAUC(spos_label==c,PM_sm_spos(c,:)');
        AT_auc_spos(n,c) = scoreAUC(spos_label==c,AT_sm_spos(c,:)');
    end
    
    for c = 1:size(PM_sm_list,1)
        PM_auc_list(n,c) = scoreAUC(list_label==c,PM_sm_list(c,:)');
        AT_auc_list(n,c) = scoreAUC(list_label==c,AT_sm_list(c,:)');
    end
    
    if size(PM_sm_sess,1)>1
        for c = 1:size(PM_sm_sess,1)
            PM_auc_sess(n,c) = scoreAUC(sess_label==c,PM_sm_sess(c,:)');
            AT_auc_sess(n,c) = scoreAUC(sess_label==c,AT_sm_sess(c,:)');
        end
    end

end

%write output hdf5
out_file = [out_dir '/' subject '_enc_pos_PMAT.hdf5'];

% set up output file
if ~exist(out_file,'file')
    h5create(out_file,'/PM_sm_spos', size(PM_sm_spos), 'Datatype','single');
    h5create(out_file,'/AT_sm_spos', size(AT_sm_spos), 'Datatype','single');
    h5create(out_file,'/PM_auc_spos', size(PM_auc_spos), 'Datatype','single');
    h5create(out_file,'/AT_auc_spos', size(AT_auc_spos), 'Datatype','single');
    
    h5create(out_file,'/PM_sm_list', size(PM_sm_list), 'Datatype','single');
    h5create(out_file,'/AT_sm_list', size(AT_sm_list), 'Datatype','single');
    h5create(out_file,'/PM_auc_list', size(PM_auc_list), 'Datatype','single');
    h5create(out_file,'/AT_auc_list', size(AT_auc_list), 'Datatype','single');
    
    h5create(out_file,'/PM_sm_sess', size(PM_sm_sess), 'Datatype','single');
    h5create(out_file,'/AT_sm_sess', size(AT_sm_sess), 'Datatype','single');
    h5create(out_file,'/PM_auc_sess', size(PM_auc_sess), 'Datatype','single');
    h5create(out_file,'/AT_auc_sess', size(AT_auc_sess), 'Datatype','single');
    
    h5create(out_file,'/serialpos', size(serialpos));
    h5create(out_file,'/list', size(list));
    h5create(out_file,'/session', size(session));
end

hdf5write(out_file,'/PM_sm_spos', single(PM_sm_spos));
hdf5write(out_file,'/AT_sm_spos', single(AT_sm_spos), 'WriteMode','append');
hdf5write(out_file,'/PM_auc_spos', single(PM_auc_spos), 'WriteMode','append');
hdf5write(out_file,'/AT_auc_spos', single(AT_auc_spos), 'WriteMode','append');

hdf5write(out_file,'/PM_sm_list', single(PM_sm_list), 'WriteMode','append');
hdf5write(out_file,'/AT_sm_list', single(AT_sm_list), 'WriteMode','append');
hdf5write(out_file,'/PM_auc_list', single(PM_auc_list), 'WriteMode','append');
hdf5write(out_file,'/AT_auc_list', single(AT_auc_list), 'WriteMode','append');

hdf5write(out_file,'/PM_sm_sess', single(PM_sm_sess), 'WriteMode','append');
hdf5write(out_file,'/AT_sm_sess', single(AT_sm_sess), 'WriteMode','append');
hdf5write(out_file,'/PM_auc_sess', single(PM_auc_sess), 'WriteMode','append');
hdf5write(out_file,'/AT_auc_sess', single(AT_auc_sess), 'WriteMode','append');

hdf5write(out_file,'/serialpos', serialpos, 'WriteMode','append');
hdf5write(out_file,'/list', list, 'WriteMode','append');
hdf5write(out_file,'/session', session, 'WriteMode','append');

end