function eval_serialpos_reinstatement(subject, pow_dir, model_dir, out_dir, rois_L, rois_R)
%EVAL_SERIALPOS_REINSTATEMENT evaluates trained serial position model to 
% decode recall period data
%
% INPUTS
% 
% subject: string - subject identifier
% pow_dir: string - path to directory containing power hdf5s for each
%                   subject
% model_dir: string - path to directory containing trained category models
%                     for each subject 
% out_dir: string - path to directory to write output hdf5s
% rois_L: string - cell array of strings, specifying freesurfer label
%                  files in the left hemisphere (see
%                  find_network_electrode.m)
% rois_R: string - cell array of strings, specifying freesurfer label
%                  files in the right hemisphere (see
%                  find_network_electrode.m)
%
% OUTPUTS
%
% none, this function will write outputs to disk in out_dir

% By James Kragel, 2021
% Distributed under Open Source BSD License

%% prep power and event information for this subject

session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

try
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item');
catch
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item_name');
end
list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');
serialpos = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/serialpos');
type = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/type');
% pirt = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/pirt');
% repeat = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/repeat');

rectime = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/rectime');
intrusion = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/intrusion');
category = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/category');

try
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/categoryNum');
catch
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/category_num');
end
% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');

% filter data/events by correct recalls that we want to analyze
type = deblank(type);

use_idx = strcmp(type,'REC_WORD') & intrusion == 0 & list > -1;
pow_all = pow_all(:,use_idx,:,:);

% update size of relevent variables
rectime = rectime(use_idx);
session = session(use_idx);
item = item(use_idx);
category = category(use_idx);
category_num = category_num(use_idx);
list = list(use_idx);
serialpos = serialpos(use_idx); % check this is correct for recalled items

% compute IRT
u_lists = unique([session list],'rows');
pirt = nan(size(session));
for ul = 1:size(u_lists,1)
    idx = session == u_lists(ul,1) & list == u_lists(ul,2);
    pirt(idx) = [rectime(find(idx,1,'first')); diff(rectime(idx))];
end

% compute repeat
for ul = 1:size(u_lists)
    idx = session == u_lists(ul,1) & list == u_lists(ul,2);
    list_items = item(idx);
    repeat_tl = false(size(list_items));
    for k = 1:length(list_items)
        if any(strcmp(list_items(1:k-1),list_items{k}))
            repeat_tl(k) = true;
        end
    end
    repeat(idx) = repeat_tl;
end
repeat = repeat';
use_idx = pirt > 1500 & repeat == 0;
pow_all = pow_all(:,use_idx,:,:);

% update size of relevent variables
session = session(use_idx);
list = list(use_idx);
serialpos = serialpos(use_idx);


% zscore by session
for s = 1:length(us)
    pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

%% filter pow_all by the electrodes near the networks we are interested in
[AT, PM] = find_network_electrode(pow_dir, subject, rois_L, rois_R);

pow_PM = pow_all(:,:,PM,:);
pow_AT = pow_all(:,:,AT,:);

if sum(PM) < 5 || sum(AT) < 5 % need at least 5 electrodes in each network
    return
end

% now, test the performance of the model

test_indices = h5read([model_dir '/' subject '_model.hdf5'],'/test_indices');
model_weights =  h5read([model_dir '/' subject '_model.hdf5'],'/model_weight');
model_vecs =  h5read([model_dir '/' subject '_model.hdf5'],'/vecs');
model_item = deblank(h5read([model_dir '/' subject '_model.hdf5'],'/item'));
model_item = strrep(model_item,'ax','axe');

best_index = h5read([model_dir '/' subject '_model.hdf5'],'/best_index');
best_mse = h5read([model_dir '/' subject '_model.hdf5'],'/best_mse');
best_param = h5read([model_dir '/' subject '_model.hdf5'],'/best_param');

spos_idx = 1:length(unique(serialpos));
list_idx = spos_idx(end)+1:spos_idx(end)+length(unique(list));
sess_idx = list_idx(end)+1:list_idx(end)+length(unique(session));

X = model_vecs;


pred_activity_PM = nan(5,8,size(pow_PM,3),size(X,1));
pred_activity_AT = nan(5,8,size(pow_AT,3),size(X,1));
for fold = 1:5
    
    % get model weights for each network, for this fold
    model_weights_PM = squeeze(model_weights(fold,:,PM,:,:));
    model_weights_AT = squeeze(model_weights(fold,:,AT,:,:));   
     
    for val = 1:size(X,1)
        for e = 1:size(pow_PM,3)
            for f = 1:size(pow_PM,4)
                pred_activity_PM(fold,f,e,val) = ...
                    squeeze(model_weights_PM(f,e,:,best_index(fold,f,e)))' * X(val,:)';
            end
        end
        
        for e = 1:size(pow_AT,3)
            for f = 1:size(pow_AT,4)
                pred_activity_AT(fold,f,e,val) = ...
                    squeeze(model_weights_AT(f,e,:,best_index(fold,f,e)))' * X(val,:)';
            end
        end
    end
    
end

pred_activity_PM = squeeze(nanmean(pred_activity_PM));
pred_activity_PM = permute(pred_activity_PM,[3 2 1]);

pred_activity_AT = squeeze(nanmean(pred_activity_AT));
pred_activity_AT = permute(pred_activity_AT,[3 2 1]);
% correlate predicted activity to observed activity across recalls

pow_PM = permute(pow_PM,[2 1 3 4]);
pow_AT = permute(pow_AT,[2 1 3 4]);

pow_PM = permute(pow_PM, [1 3 4 2]);
pow_AT = permute(pow_AT, [1 3 4 2]);


% average over predicted activity to create marginal means 

% spos
pow_pred_PM_spos = nan(length(spos_idx),size(pred_activity_PM,2),size(pred_activity_PM,3));
for s = 1:length(spos_idx)
     pow_pred_PM_spos(s,:,:) = squeeze(nanmean(pred_activity_PM(model_vecs(:,spos_idx(s))==1,:,:)));
end

pow_pred_AT_spos = nan(length(spos_idx),size(pred_activity_AT,2),size(pred_activity_AT,3));
for s = 1:length(spos_idx)
     pow_pred_AT_spos(s,:,:) = squeeze(nanmean(pred_activity_AT(model_vecs(:,spos_idx(s))==1,:,:)));
end

% list
pow_pred_PM_list = nan(length(list_idx),size(pred_activity_PM,2),size(pred_activity_PM,3));
for s = 1:length(list_idx)
     pow_pred_PM_list(s,:,:) = squeeze(nanmean(pred_activity_PM(model_vecs(:,list_idx(s))==1,:,:)));
end

pow_pred_AT_list = nan(length(list_idx),size(pred_activity_AT,2),size(pred_activity_AT,3));
for s = 1:length(list_idx)
     pow_pred_AT_list(s,:,:) = squeeze(nanmean(pred_activity_AT(model_vecs(:,list_idx(s))==1,:,:)));
end

% sess
pow_pred_PM_sess = nan(length(sess_idx),size(pred_activity_PM,2),size(pred_activity_PM,3));
for s = 1:length(sess_idx)
     pow_pred_PM_sess(s,:,:) = squeeze(nanmean(pred_activity_PM(model_vecs(:,sess_idx(s))==1,:,:)));
end

pow_pred_AT_sess = nan(length(sess_idx),size(pred_activity_AT,2),size(pred_activity_AT,3));
for s = 1:length(sess_idx)
     pow_pred_AT_sess(s,:,:) = squeeze(nanmean(pred_activity_AT(model_vecs(:,sess_idx(s))==1,:,:)));
end



n_elec = 5;
n_it = 1000;

[~,~,spos_label] = unique(serialpos);
[~,~,list_label] = unique(list);
[~,~,sess_label] = unique(session);


AT_corr_spos = nan(n_it, size(pow_AT,1), length(unique(serialpos)), size(pow_AT,4));
PM_corr_spos = nan(n_it, size(pow_PM,1), length(unique(serialpos)), size(pow_PM,4));

AT_corr_list = nan(n_it, size(pow_AT,1), length(unique(list)), size(pow_AT,4));
PM_corr_list = nan(n_it, size(pow_PM,1), length(unique(list)), size(pow_PM,4));

AT_corr_sess = nan(n_it, size(pow_AT,1), length(unique(session)), size(pow_AT,4));
PM_corr_sess = nan(n_it, size(pow_PM,1), length(unique(session)), size(pow_PM,4));


PM_auc_spos = nan(n_it, length(unique(serialpos)), size(pow_PM,4));
AT_auc_spos = nan(n_it, length(unique(serialpos)), size(pow_AT,4));

PM_auc_list = nan(n_it, length(unique(list)), size(pow_PM,4));
AT_auc_list = nan(n_it, length(unique(list)), size(pow_AT,4));

PM_auc_sess = nan(n_it, length(unique(session)), size(pow_PM,4));
AT_auc_sess = nan(n_it, length(unique(session)), size(pow_AT,4));

for n = 1:n_it
    
    PM_idx = randperm(sum(PM)); PM_idx = PM_idx(1:n_elec);
    AT_idx = randperm(sum(AT)); AT_idx = AT_idx(1:n_elec);
    
    % spos
    for t = 1:size(pow_PM,4)
        tmp1 = reshape(pow_pred_PM_spos(:,PM_idx,:),size(pow_pred_PM_spos,1),[]);
        tmp2 = reshape(pow_PM(:,PM_idx,:,t),size(pow_PM,1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        PM_corr_spos(n,:,:,t) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end
    
    for t = 1:size(pow_AT,4)
        tmp1 = reshape(pow_pred_AT_spos(:,AT_idx,:),size(pow_pred_AT_spos,1),[]);
        tmp2 = reshape(pow_AT(:,AT_idx,:,t),size(pow_AT,1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        AT_corr_spos(n,:,:,t) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end
    
    % list
    for t = 1:size(pow_PM,4)
        tmp1 = reshape(pow_pred_PM_list(:,PM_idx,:),size(pow_pred_PM_list,1),[]);
        tmp2 = reshape(pow_PM(:,PM_idx,:,t),size(pow_PM,1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        PM_corr_list(n,:,:,t) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end
    
    for t = 1:size(pow_AT,4)
        tmp1 = reshape(pow_pred_AT_list(:,AT_idx,:),size(pow_pred_AT_list,1),[]);
        tmp2 = reshape(pow_AT(:,AT_idx,:,t),size(pow_AT,1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        AT_corr_list(n,:,:,t) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end
    
    % sess
    for t = 1:size(pow_PM,4)
        tmp1 = reshape(pow_pred_PM_sess(:,PM_idx,:),size(pow_pred_PM_sess,1),[]);
        tmp2 = reshape(pow_PM(:,PM_idx,:,t),size(pow_PM,1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        PM_corr_sess(n,:,:,t) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end
    
    for t = 1:size(pow_AT,4)
        tmp1 = reshape(pow_pred_AT_sess(:,AT_idx,:),size(pow_pred_AT_sess,1),[]);
        tmp2 = reshape(pow_AT(:,AT_idx,:,t),size(pow_AT,1),[]);
        tmp_corr = corr(vertcat(tmp1,tmp2)','rows','complete');
        AT_corr_sess(n,:,:,t) = tmp_corr(1:size(tmp1,1),size(tmp1,1)+1:end)';
    end
        
    % TODO: init sms
    PM_sm_spos = nan(40,size(PM_corr_spos,3), size(PM_corr_spos,2));
    AT_sm_spos = nan(40,size(AT_corr_spos,3), size(AT_corr_spos,2));
    
    PM_sm_list = nan(40,size(PM_corr_list,3), size(PM_corr_list,2));
    AT_sm_list = nan(40,size(AT_corr_list,3), size(AT_corr_list,2));
    
    PM_sm_sess = nan(40,size(PM_corr_sess,3), size(PM_corr_sess,2));
    AT_sm_sess = nan(40,size(AT_corr_sess,3), size(AT_corr_sess,2));
    
    for t = 1:40
        PM_sm_spos(t,:,:) = softmax(squeeze(PM_corr_spos(n,:,:,t))');
        AT_sm_spos(t,:,:) = softmax(squeeze(AT_corr_spos(n,:,:,t))');
        
        PM_sm_list(t,:,:) = softmax(squeeze(PM_corr_list(n,:,:,t))');
        AT_sm_list(t,:,:) = softmax(squeeze(AT_corr_list(n,:,:,t))');
        
        PM_sm_sess(t,:,:) = softmax(squeeze(PM_corr_sess(n,:,:,t))');
        AT_sm_sess(t,:,:) = softmax(squeeze(AT_corr_sess(n,:,:,t))');
    end
 
    for c = 1:size(PM_sm_spos,2)
        for t = 1:40
            PM_auc_spos(n,c,t) = scoreAUC(spos_label==c,squeeze(PM_sm_spos(t,c,:)));
            AT_auc_spos(n,c,t) = scoreAUC(spos_label==c,squeeze(AT_sm_spos(t,c,:)));
        end
    end
    
    for c = 1:size(PM_sm_list,2)
        for t = 1:40
            PM_auc_list(n,c,t) = scoreAUC(list_label==c,squeeze(PM_sm_list(t,c,:)));
            AT_auc_list(n,c,t) = scoreAUC(list_label==c,squeeze(AT_sm_list(t,c,:)));
        end
    end
    
    if size(PM_sm_sess,2)>1
        for c = 1:size(PM_sm_sess,2)
            for t = 1:40
            PM_auc_sess(n,c,t) = scoreAUC(sess_label==c,squeeze(PM_sm_sess(t,c,:)));
            AT_auc_sess(n,c,t) = scoreAUC(sess_label==c,squeeze(AT_sm_sess(t,c,:)));
            end
        end
    end
end
%write output hdf5
out_file = [out_dir '/' subject '_reins_pos_PMAT.hdf5'];

if ~exist(out_file,'file')
    h5create(out_file,'/PM_corr_spos', size(PM_corr_spos), 'Datatype','single');
    h5create(out_file,'/AT_corr_spos', size(AT_corr_spos), 'Datatype','single');
    h5create(out_file,'/PM_auc_spos', size(PM_auc_spos), 'Datatype','single');
    h5create(out_file,'/AT_auc_spos', size(AT_auc_spos), 'Datatype','single');
    
    h5create(out_file,'/PM_corr_list', size(PM_corr_list), 'Datatype','single');
    h5create(out_file,'/AT_corr_list', size(AT_corr_list), 'Datatype','single');
    h5create(out_file,'/PM_auc_list', size(PM_auc_list), 'Datatype','single');
    h5create(out_file,'/AT_auc_list', size(AT_auc_list), 'Datatype','single');
    
    h5create(out_file,'/PM_corr_sess', size(PM_corr_sess), 'Datatype','single');
    h5create(out_file,'/AT_corr_sess', size(AT_corr_sess), 'Datatype','single');
    h5create(out_file,'/PM_auc_sess', size(PM_auc_sess), 'Datatype','single');
    h5create(out_file,'/AT_auc_sess', size(AT_auc_sess), 'Datatype','single');
    
    h5create(out_file,'/serialpos', size(serialpos));
    h5create(out_file,'/list', size(list));
    h5create(out_file,'/session', size(session));
end

hdf5write(out_file,'/PM_corr_spos', single(PM_corr_spos));
hdf5write(out_file,'/AT_corr_spos', single(AT_corr_spos), 'WriteMode','append');
hdf5write(out_file,'/PM_auc_spos', single(PM_auc_spos), 'WriteMode','append');
hdf5write(out_file,'/AT_auc_spos', single(AT_auc_spos), 'WriteMode','append');

hdf5write(out_file,'/PM_corr_list', single(PM_corr_list), 'WriteMode','append');
hdf5write(out_file,'/AT_corr_list', single(AT_corr_list), 'WriteMode','append');
hdf5write(out_file,'/PM_auc_list', single(PM_auc_list), 'WriteMode','append');
hdf5write(out_file,'/AT_auc_list', single(AT_auc_list), 'WriteMode','append');

hdf5write(out_file,'/PM_corr_sess', single(PM_corr_sess), 'WriteMode','append');
hdf5write(out_file,'/AT_corr_sess', single(AT_corr_sess), 'WriteMode','append');
hdf5write(out_file,'/PM_auc_sess', single(PM_auc_sess), 'WriteMode','append');
hdf5write(out_file,'/AT_auc_sess', single(AT_auc_sess), 'WriteMode','append');

hdf5write(out_file,'/serialpos', serialpos, 'WriteMode','append');
hdf5write(out_file,'/list', list, 'WriteMode','append');
hdf5write(out_file,'/session', session, 'WriteMode','append');

end