function [AT, PM, LR, xyz] = find_network_electrode(pow_dir, subject, ...
    rois_L, rois_R, good_elec_flag)
%FIND_NETWORK_ELECTRODE finds electrodes within PM and AT networks for a
% given subject
%
% INPUTS
% 
% pow_dir: string - path to directory containing power hdf5s for each
%                   subject
% subject: string - subject identifier
% rois_L: string - cell array of strings, specifying freesurfer label
%                  files in the left hemisphere
% rois_R: string - cell array of strings, specifying freesurfer label
%                  files in the right hemisphere 
% good_elec_flag: logical - flag indicating whether to only include 'good'
%                           electrodes
%
%
% OUTPUTS
%
% AT: logical - array indicating whether a contact is near the AT network
% PM: logical - array indicating whether a contact is near the PM network
% LR: logical - array indicating whether in the left (1) or right (0) hemi
% xyz: array of coordinates for each contact

% By James Kragel, 2021
% Distributed under Open Source BSD License

% Example ROIs:
% rois_L = {'L_PGp_ROIs.label', 'L_TGd_ROIs.label'};
% rois_R = {'R_PGp_ROIs.label', 'R_TGd_ROIs.label'};

% Some hard-coded paths, these may need to change
label_dir = '..\..\data\labels\'; %.label files 
pairs_dir = '..\..\data\localizations\'; %pairs.json files 
surf_dir = '\path\to\freesurfer\subjects\average\surf\'; %lh rh.pial

if ~exist('good_elec_flag','var')
    good_elec_flag = false;
end

info = h5info([pow_dir '/' subject '_pow.hdf5']);
gnames = {info.Groups.Name};
has_tal_struct = any(strcmp('/tal_struct',gnames));

if has_tal_struct
    x = h5read([pow_dir '/' subject '_pow.hdf5'],'/tal_struct/atlases/avg.dural/x');
    y = h5read([pow_dir '/' subject '_pow.hdf5'],'/tal_struct/atlases/avg.dural/y');
    z = h5read([pow_dir '/' subject '_pow.hdf5'],'/tal_struct/atlases/avg.dural/z');
    
    % read from json to deal w/ good elecs
    tal = get_tal_from_json([pairs_dir subject '\pairs.json'], good_elec_flag, 'pairs');
    
    if good_elec_flag
        % reorder new tal struct to match xyz in hdf5
        rot_idx = zeros(1,length(tal));
        tmp = cat(1,tal.xyz_avg);
        for k = 1:length(rot_idx)
            tmp2 = find(x(k) == tmp(:,1) & y(k) == tmp(:,2) & z(k) == tmp(:,3));
            if size(tmp2,1) ~=1
                rot_idx(k) = nan;
            else
                rot_idx(k) = find(x(k) == tmp(:,1) & y(k) == tmp(:,2) & z(k) == tmp(:,3));
            end
        end
        
        bad_elec = false(size(x));
        if isfield(tal(1),'bad_elec')
            for k = 1:length(rot_idx)
                if ~isnan(rot_idx)
                    bad_elec(rot_idx(k))=tal(k).bad_elec;
                end
            end
        end
        % nan coordinates to avoid network membership
        x(bad_elec) = nan;
        y(bad_elec) = nan;
        z(bad_elec) = nan;
    end
    
else % don't have avg.dural in power json, so look up on rhino
    
    contact_name = h5read([pow_dir '/' subject '_pow.hdf5'],'/h5info/bipolar_info/contact_name');
    montage = unique(h5read([pow_dir '/' subject '_pow.hdf5'],'/events/montage'));
    contact_name = deblank(contact_name);
    montage = deblank(montage);
    
    tal = get_tal_from_json([pairs_dir subject '\pairs.json'], good_elec_flag, 'pairs');
    
    for t = 1:length(tal)
        tal(t).tagName = upper(tal(t).tagName);
    end
    
    xyz = nan(length(contact_name),3);
    for t = 1:length(contact_name)
        if ismember(contact_name{t}, {tal.tagName})
            xyz(t,:) = vertcat(tal(ismember({tal.tagName},contact_name{t})).xyz_avg);
        end
    end
    
    x = xyz(:,1); y = xyz(:,2); z = xyz(:,3);
    
    if good_elec_flag
        bad_elec = [tal.bad_elec];
        % nan coordinates to avoid network membership
        x(bad_elec) = nan;
        y(bad_elec) = nan;
        z(bad_elec) = nan;
    end
end

LR = false(size(x));
LR(x<0) = true;

[~, s_L_PM] = read_label([label_dir rois_L{1}]);
[~, s_L_AT] = read_label([label_dir rois_L{2}]);

[~, s_R_PM] = read_label([label_dir rois_R{1}]);
[~, s_R_AT] = read_label([label_dir rois_R{2}]);

% read surface data
surfL = fullfile(surf_dir,'lh.pial');
vL = read_surf_wrapper(surfL);
surfR = fullfile(surf_dir,'rh.pial');
vR = read_surf_wrapper(surfR);

% for each electrode, find closest network, and the distance
xyz = [x y z];
xyz_L = xyz(x<0,:,:);
xyz_R = xyz(x>0,:,:);

dist_L_PM = network_membership(xyz_L, vL, s_L_PM);
dist_L_AT = network_membership(xyz_L, vL, s_L_AT);

dist_L_diff = abs(dist_L_PM - dist_L_AT);
L_PM = dist_L_PM < 8 & dist_L_diff > 4;
L_AT = dist_L_AT < 8 & dist_L_diff > 4;

dist_R_PM = network_membership(xyz_R, vR, s_R_PM);
dist_R_AT = network_membership(xyz_R, vR, s_R_AT);

dist_R_diff = abs(dist_R_PM - dist_R_AT);
R_PM = dist_R_PM < 8 & dist_R_diff > 4;
R_AT = dist_R_AT < 8 & dist_R_diff > 4;

PM = false(size(x));
PM(x<0) = L_PM;
PM(x>0) = R_PM;

AT = false(size(x));
AT(x<0) = L_AT;
AT(x>0) = R_AT;


end

function [dist, idx] = network_membership(xyz, v, s)

dist = nan(size(xyz,1),1);
idx = nan(size(xyz,1),1);

for e = 1:size(xyz,1)
    
    ref = repmat(xyz(e,:),size(v,1),1);
    tmp_dists = sqrt(sum((ref-v).^2,2));
    
    dist(e) = min(tmp_dists(~isnan(s)));
    idx(e) = find(tmp_dists==min(tmp_dists)); %ties?
    
end

end


function [vertex, scalar] = read_label(labelfile)

fid = fopen(labelfile);
line = fgets(fid) ;
nv = sscanf(line, '%d') ;
l = fscanf(fid, '%d %f %f %f %f\n') ;
l = reshape(l, 5, nv) ;
l = l' ;

fclose(fid) ;

vertex = l(:,1);
scalar = l(:,5);

end


