function [p_temp, p_cat, z_temp, z_cat] = run_behav_control(t, c)
%RUN_BEHAV_CONTROL trains encoding model from word2vec category reps
%
% INPUTS
%
% t: temporal factor, per subject
% c: category factor, per subject
% p: probability of recall, per subject
%
% OUTPUTS
%
% p_temp: p-value for control analysis per subject, temporal organization
% p_cat: p-value for control analysis per subject, temporal organization
% z_temp: standardized temporal organization, accounting for list
% z_cat: standardized category organization, accounting for list

% By James Kragel, 2021
% Distributed under Open Source BSD License

subjects = ident_mscript_subs;

p_temp = nan(size(subjects));
p_cat = nan(size(subjects));
z_temp = nan(size(subjects));
z_cat = nan(size(subjects));


parfor s = 1:length(subjects)
    subj = subjects{s};
    
   [p_temp(s), p_cat(s), z_temp(s), z_cat(s)] = gen_null_seq(subj, ...
                                                             t(s), ...
                                                             c(s)); 
end

end

function [p_temp, p_cat, z_temp, z_cat] = gen_null_seq(subj, t, c)
%GEN_NULL_SEQ from random retrieval, generate null distributions with a
%given tf or cf score

load(['data/' subj '_data.mat'],'data');

% build dist mat from data.pres_itemnos

tmp = data.pres_itemnos;
tmp(tmp<=0)=nan;

% ignore incomplete trials
ig_idx = any(isnan(tmp),2);
data.pres_itemnos(ig_idx,:)=nan;
data.rec_itemnos(ig_idx,:)=nan;
data.recalls(ig_idx,:)=nan;

data.pres_itemnos = tmp;
data.rec_itemnos(data.rec_itemnos==0)=nan;

[ui, ia] = unique(tmp);
try
    uc = data.pres.categoryNum(ia);
catch
    uc = data.pres.category_num(ia);
end

nan_idx = isnan(ui);
ui(nan_idx) = [];
uc(nan_idx) = [];

dist_mat = nan(length(ui));
for i = 1:length(ui)
    for j = 1:length(ui)
        if uc(j) == uc(i)
            dist_mat(ui(i),ui(j)) = 1;
        else
            dist_mat(ui(i),ui(j)) = 0;
        end
    end
end

n_rec = 12*p_rec(data.recalls, 1:size(data.recalls,1), 12);
seqs = nan(5000, length(n_rec), max(n_rec));
tf = nan(5000,length(n_rec));
cf = nan(5000,length(n_rec));

for n = 1:5000
    for l = 1:length(n_rec)
        tmp = randperm(12,n_rec(l));
        seqs(n,l,1:length(tmp)) = tmp;
        
        tf(n,l) = temp_fact(squeeze(seqs(n,l,:))', 1, 12);
        cf(n,l) = dist_fact(data.pres_itemnos(l,squeeze(seqs(n,l,1:length(tmp)))'), ...
            data.pres_itemnos(l,:), ...
            1, dist_mat);
    end
end

% find 1000 best trials to match either tf or cf

% compute null cf
t_by_trial = temp_fact(data.recalls, 1:size(data.recalls,1), 12);
idx = nan(length(t_by_trial), 1000);
null_cf = idx;
for i = 1:length(t_by_trial)
    [sort_tf,j] = sort(abs(tf(:,i)-t_by_trial(i)));
    if all(isnan(sort_tf)) % no recalls
        continue
    end
    idx(i,:) = j(1:1000);
    null_cf(i,:)  = cf(j(1:1000),i);
end

% compute null tf
c_by_trial = dist_fact(data.rec_itemnos, ...
            data.pres_itemnos, ...
            1:size(data.recalls,1), dist_mat);

idx = nan(length(c_by_trial), 1000);
null_tf = idx;
for i = 1:length(c_by_trial)
    [sort_cf,j] = sort(abs(cf(:,i)-c));%-c_by_trial(i)));
    if all(isnan(sort_cf)) % no recalls
        continue
    end
    idx(i,:) = j(1:1000);
    null_tf(i,:)  = tf(j(1:1000),i);
end

p_temp = (1 + sum(null_tf(:) > t))/(sum(~isnan(null_tf(:))+1));
p_cat = (1 + sum(null_cf(:) > c))/(sum(~isnan(null_cf(:))+1));

z_temp = (t-nanmean(null_tf(:)))/nanstd(null_tf(:));
z_cat  = (c-nanmean(null_cf(:)))/nanstd(null_cf(:));

end


