function train_category_model(subject, pow_dir, res_dir, cat_flag)
%TRAIN_CATEGORY_MODEL trains encoding model from word2vec category reps
%
% INPUTS
%
% subject: string - subject identifier
% pow_dir: string - path to directory containing encoding
%                       power hdf5s for each subject
% res_dir: string - path to directory to save model
% cat_flag: logical - boolean specifying where to average across all items
%                     in a category
%
% OUTPUTS
%
% none: this function write out an hdf5 file with model information in 
% res_dir

% By James Kragel, 2021
% Distributed under Open Source BSD License

% get semantic information
words = textread('../../data/CatFR1_wordpool.txt','%s');
vecs = h5read('../../data/CatFR1_vecs.h5','/vecs');

% get data for this subject
session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

try
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item');
catch
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item_name');
end
item  = deblank(lower(item));
item = strrep(item,'axe','ax');

category = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/category');
try
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/categoryNum');
catch
    category_num = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/category_num');
end

% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');
list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');

pow_all(:,list<0,:,:) = [];
item(list<0)=[];
session(list<0) = [];
category_num(list<0) = [];

[ui,i] = unique(item);
ucbi = category_num(i);

% zscore by session
for s = 1:length(us)
    pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

pow_all = squeeze(nanmean(pow_all(27:104,:,:,:))); % average over time
pow_size = size(pow_all);

% set up output file
if ~exist([res_dir '/' subject '_model.hdf5'],'file')
    h5create([res_dir '/' subject '_model.hdf5'],'/item', length(item));
    h5create([res_dir '/' subject '_model.hdf5'],'/session', length(session));
end

hdf5write([res_dir '/' subject '_model.hdf5'], '/session', session);
hdf5write([res_dir '/' subject '_model.hdf5'], '/item', item, 'WriteMode','append');

% set up folds
indices = crossvalind('Kfold',length(ui),5);

hdf5write([res_dir '/' subject '_model.hdf5'], '/folds', indices, 'WriteMode','append');

% build base design matrix
X = nan(pow_size(1),300);
for i = 1:pow_size(1)
    if cat_flag
        X(i,:) = nanmean(vecs(ucbi==category_num(i),:)); % category mean model
    else
        X(i,:) = vecs(strcmp(words,item{i}),:); % item-level model
    end
end
h5create([res_dir '/' subject '_model.hdf5'], '/vecs', size(X),'Datatype', 'single')
h5write([res_dir '/' subject '_model.hdf5'], '/vecs', single(X));

k_par = logspace(log10(1/10^5),log10(10^5),50);

h5create([res_dir '/' subject '_model.hdf5'],'/model_weight',...
    [5, pow_size(3), pow_size(2), 300, length(k_par)],'Datatype','single')

h5create([res_dir '/' subject '_model.hdf5'],'/train_indices',...
    [5, length(item)]);

h5create([res_dir '/' subject '_model.hdf5'],'/test_indices',...
    [5, length(item)]);

h5create([res_dir '/' subject '_model.hdf5'],...
    '/best_param',...
    [5 pow_size(3) pow_size(2)]);

h5create([res_dir '/' subject '_model.hdf5'],...
    '/best_index',...
    [5 pow_size(3) pow_size(2)]);

h5create([res_dir '/' subject '_model.hdf5'],...
    '/best_mse',...
    [5 pow_size(3) pow_size(2)]);

for fold = 1:5
    
    train_indices = ismember(item, ui(~ismember(indices,fold)));
    test_indices = ismember(item, ui(ismember(indices,fold)));
    
    h5write([res_dir '/' subject '_model.hdf5'],'/train_indices',single(train_indices)',[fold 1],[1 length(train_indices)]);
    h5write([res_dir '/' subject '_model.hdf5'],'/test_indices',single(test_indices)',[fold 1],[1 length(test_indices)]);
    
    % train
    
    [B1, Y_hat, best_index, best_param, best_mse] = train_predict_ridge(pow_all,...
        pow_size, train_indices, X);
    
    h5write([res_dir '/' subject '_model.hdf5'],...
        '/model_weight',...
        single(reshape(B1,[1 size(B1,1) size(B1,2) size(B1,3) size(B1,4)])),...
        [fold 1 1 1 1],...
        [1 size(B1,1) size(B1,2) size(B1,3) size(B1,4)]);
        
        % fold by channel by freq by feature x penalty param
    
    h5write([res_dir '/' subject '_model.hdf5'],...
        '/best_index',...
        reshape(single(best_index),[1 size(best_index,1) size(best_index,2)]),...
        [fold 1 1],...
        [1 pow_size(3) pow_size(2)]);
    
    h5write([res_dir '/' subject '_model.hdf5'],...
        '/best_param',...
        reshape(single(best_param),[1 size(best_param,1) size(best_param,2)]),...
        [fold 1 1],...
        [1 pow_size(3) pow_size(2)]);

    h5write([res_dir '/' subject '_model.hdf5'],...
        '/best_mse',...
        reshape(single(best_mse),[1 size(best_mse,1) size(best_mse,2)]),...
        [fold 1 1],...
        [1 pow_size(3) pow_size(2)]);
    
    test_pow_all = pow_all(test_indices,:,:);
    test_items = item(test_indices);
    test_cats = category_num(test_indices);
    Y_hat_test = Y_hat(:,:,test_indices);
    [unique_test_items, i] = unique(test_items);
    unique_test_cats = test_cats(i);
    Y_hat_test_unique = Y_hat_test(:,:,i);
    
    % compute confusion matrix for this fold
    pred_cats = unique_test_cats;
    test_pow = test_pow_all;
    pred_pow = Y_hat_test_unique;
    class_rate = compute_confusion(test_pow,pred_pow,test_cats,pred_cats);
    
    test_pow_avg = nan(size(test_pow_all,3),size(test_pow_all,2),length(unique_test_items));
    for ut = 1:length(unique_test_items)
        if sum(strcmp(test_items,unique_test_items{ut}))==1
            test_pow_avg(:,:,ut) = squeeze((test_pow_all(strcmp(test_items,unique_test_items{ut}),:,:)))';
        else
            test_pow_avg(:,:,ut) = squeeze(nanmean(test_pow_all(strcmp(test_items,unique_test_items{ut}),:,:)))';
        end
    end
    
    test_corr = nan(size(test_pow_avg,3),size(Y_hat_test_unique,3));
    for i = 1:size(test_pow_avg,3) % test on each observation
        test_pat = test_pow_avg(:,:,i); test_pat = test_pat(:);
        
        % compute correlation with predictions
        for k = 1:size(Y_hat_test_unique,3)
            pred_pat = Y_hat_test_unique(:,:,k); pred_pat = pred_pat(:);
            test_corr(i,k) = corr(test_pat,pred_pat);
        end
    end
    
    % compute ranks of correlations - item level
    for k = 1:size(test_corr,1)
        ranked_corr(k) = percentile_rank(test_corr(k,k),test_corr(k,:));
    end
    
    % compute ranks of correlations - category level (for each item,
    % exclude all other items of the same category)
    for k = 1:size(test_corr,1)
        ranked_corr_cat(k) = percentile_rank(test_corr(k,k),...
            [test_corr(k,k) test_corr(k,~ismember(unique_test_cats,unique_test_cats(k)))]);
    end
    
    % compute null rankings
    null_ranked_corr = nan(size(test_corr,1),1000);
    null_ranked_corr_cat = nan(size(test_corr,1),1000);
    for k = 1:size(test_corr,1)
        parfor n = 1:1000
            null_corr = test_corr; null_corr = null_corr(:,randperm(size(null_corr,2)));
            null_ranked_corr(k,n) = percentile_rank(null_corr(k,k),null_corr(k,:));
            null_ranked_corr_cat(k,n) = percentile_rank(null_corr(k,k),...
                [null_corr(k,k) null_corr(k,~ismember(unique_test_cats,unique_test_cats(k)))]);
        end
    end
    
    
    % test - just want to compute performance on the test set items
    accuracy = nanmean(ranked_corr);
    accuracy_cat = nanmean(ranked_corr_cat);
    p = (sum(accuracy < mean(null_ranked_corr))+1)/length(null_ranked_corr);
    p_cat = (sum(accuracy_cat < mean(null_ranked_corr_cat))+1)/length(null_ranked_corr_cat);
    
    if fold == 1
        h5create([res_dir '/' subject '_model.hdf5'],'/accuracy',[1 5],'Datatype','single')
        h5create([res_dir '/' subject '_model.hdf5'],'/p',[1 5],'Datatype','single')
        h5create([res_dir '/' subject '_model.hdf5'],'/accuracy_cat',[1 5],'Datatype','single')
        h5create([res_dir '/' subject '_model.hdf5'],'/p_cat',[1 5],'Datatype','single')
        h5create([res_dir '/' subject '_model.hdf5'],'/class_rate',[25 25 5],'Datatype','single')
    end
    h5write([res_dir '/' subject '_model.hdf5'],'/accuracy', single(accuracy), [1 fold],[1 1]);
    h5write([res_dir '/' subject '_model.hdf5'],'/p', single(p), [1 fold],[1 1]);
    
    h5write([res_dir '/' subject '_model.hdf5'],'/accuracy_cat', single(accuracy_cat), [1 fold],[1 1]);
    h5write([res_dir '/' subject '_model.hdf5'],'/p_cat', single(p_cat), [1 fold],[1 1]);
    h5write([res_dir '/' subject '_model.hdf5'],'/class_rate', single(class_rate), [1 1 fold],[25 25 1]);
end

function class_rate = compute_confusion(test_pow,pred_pow,test_cats,pred_cats)

test_pow = reshape(test_pow,size(test_pow,1),[]);

pred_pow = permute(pred_pow,[3 2 1]);
pred_pow = reshape(pred_pow,size(pred_pow,1),[]);

possible_cats = 0:24;
Y = nan(size(test_pow,1),25);
T = zeros(size(test_pow,1),25);

% build full matrix
for x = 1:length(test_cats)
    for y = 1:length(pred_cats)
        Y(x,possible_cats==pred_cats(y)) = corr(test_pow(x,:)',pred_pow(y,:)');
    end
    T(x,possible_cats==test_cats(x))=1;
end

for k = 1:size(Y,2) % bound to [0 1]
    Y(:,k) = (Y(:,k)-min(Y(:,k)))/max((Y(:,k)-min(Y(:,k))));
end

% go through all pairwise combinations
combinations = combnk(1:25,2);
class_rate = nan(25);
for k = 1:size(combinations,1)
    if ~any(isnan(Y(:,combinations(k,1))) | isnan(Y(:,combinations(k,2))))
        % only use observations with the test pairs
        obs_idx = any(T(:,combinations(k,:)),2);
        C(k) = confusion(T(obs_idx,combinations(k,:))',Y(obs_idx,combinations(k,:))');
        class_rate(combinations(k,2),combinations(k,1)) = 1-C(k);
    end
end

