function [B1, Y_hat, best_index, best_param, best_mse] = train_predict_ridge(pow_all,...
    pow_size, train_indices, X)
%TRAIN_PREDICT_RIDGE fits a ridge regression model
%
% given power (Y), and a design matrix (X), and training observations
% (train indices), find the best fitting params (B1) and predicted data
% (Y_hat) by optimizing the ridge parameter
%
%
% INPUTS
%
% pow_all: array containing power
% pow_size: array with size of pow_all
% train_indices: array specifying which timepoints to train on
% X: design matrix (word2vec or time representations)
%
% OUTPUTS
%
% none: this function write out an hdf5 file with model information in 
% res_dir

% By James Kragel, 2021
% Distributed under Open Source BSD License

k_par = logspace(log10(1/10^3),log10(10^10),50);

Y_hat = nan(pow_size(3),pow_size(2),length(train_indices));
best_index = nan(pow_size(3),pow_size(2));
best_param = best_index;
best_mse = best_param;
B1 = nan(pow_size(3),pow_size(2),size(X,2),length(k_par));

for f = 1:pow_size(3)
    for c = 1:pow_size(2)
        
        % 10 fold nested cv on training data to find best ridge param
        nest_indices = crossvalind('Kfold',sum(train_indices),10);
        train_dat = pow_all(train_indices,c,f);
        train_model = X(train_indices,:);
        
        temp = nan(size(X,2),length(k_par));
        B0 = nan(size(X,2)+1,length(k_par));
        mse = nan(10,length(k_par));
        for  i = 1:10
            B_tmp = ridge(train_dat(nest_indices~=i),train_model(nest_indices~=i,:),k_par);
            
            % put back in scale of original data
            m = mean(train_model(nest_indices~=i,:));
            s = std(train_model(nest_indices~=i,:),0,1)';
            s(s==0)=eps; % to prevent nan mse 
            % compute MSE
            for k = 1:size(B_tmp,2)
                temp(:,k) = B_tmp(:,k)./s;
                B0(:,k) = [mean(train_dat(nest_indices~=i))-m*temp(:,k); temp(:,k)];
                mse(i,k) = mean((train_dat(nest_indices==i)'-(B0(1,k)+B0(2:end,k)'*train_model(nest_indices==i,:)')).^2);
            end

        end
        mse = mean(mse); % average over cv folds
        
        best_index(f,c) = find(mse==min(mse),1,'first');
        best_param(f,c) = k_par(best_index(f,c));
        best_mse(f,c) = mse(best_index(f,c));
        
        % set up parameter optimization // % train final model
        B1(f,c,:,:) = ridge(pow_all(train_indices,c,f),X(train_indices,:),k_par);
        Y_hat(f,c,:) = squeeze(B1(f,c,:,best_index(f,c)))'*X'; % include Y_hat over training and test set,
    end
   
end

end

