function train_serialpos_model(subject, pow_dir, res_dir)
%TRAIN_SERIALPOS_MODEL trains encoding model based on temporal features
%
% INPUTS
%
% subject: string - subject identifier
% pow_dir: string - path to directory containing encoding
%                       power hdf5s for each subject
% res_dir: string - path to directory to save model
%
% OUTPUTS
%
% none: this function write out an hdf5 file with model information in 
% res_dir

% By James Kragel, 2021
% Distributed under Open Source BSD License

session = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/session');
us = unique(session);

try
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item');
catch
    item = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/item_name');
end
item  = deblank(lower(item));
item = strrep(item,'axe','ax');

% load data
pow_all = h5read([pow_dir '/' subject '_pow.hdf5'],'/data');
list = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/list');
serialpos = h5read([pow_dir '/' subject '_pow.hdf5'],'/events/serialpos');

% remove practice lists
pow_all(:,list<0,:,:) = [];
item(list<0)=[];
session(list<0) = [];
serialpos(list<0) = [];
list(list<0) = [];

% zscore by session
for s = 1:length(us)
    pow_all(:,session == us(s),:,:) = zscore(pow_all(:,session == us(s),:,:),[],2);
end

pow_all = squeeze(nanmean(pow_all(27:104,:,:,:))); % average over time
pow_size = size(pow_all);

% set up output file
if ~exist([res_dir '/' subject '_model.hdf5'],'file')
    h5create([res_dir '/' subject '_model.hdf5'],'/item', length(item));
    h5create([res_dir '/' subject '_model.hdf5'],'/list', length(list));
    h5create([res_dir '/' subject '_model.hdf5'],'/session', length(session));
    h5create([res_dir '/' subject '_model.hdf5'],'/serialpos', length(serialpos));
end

hdf5write([res_dir '/' subject '_model.hdf5'], '/session', session);
hdf5write([res_dir '/' subject '_model.hdf5'], '/item', item, 'WriteMode','append');
hdf5write([res_dir '/' subject '_model.hdf5'], '/list', list, 'WriteMode','append');
hdf5write([res_dir '/' subject '_model.hdf5'], '/serialpos', serialpos, 'WriteMode','append');

% set up folds
indices = crossvalind('Kfold',length(ui),5);

hdf5write([res_dir '/' subject '_model.hdf5'], '/folds', indices, 'WriteMode','append');

X = dummyvar([double(serialpos) double(list) double(session)+1]);

nX = size(X,2);

h5create([res_dir '/' subject '_model.hdf5'],'/vecs',size(X),'Datatype','single')
h5write([res_dir '/' subject '_model.hdf5'],'/vecs',single(X));

k_par = logspace(log10(1/10^5),log10(10^5),50);

h5create([res_dir '/' subject '_model.hdf5'],'/model_weight',...
    [5, pow_size(3), pow_size(2), nX, length(k_par)],'Datatype','single')

h5create([res_dir '/' subject '_model.hdf5'],'/train_indices',...
    [5, length(item)]);

h5create([res_dir '/' subject '_model.hdf5'],'/test_indices',...
    [5, length(item)]);

h5create([res_dir '/' subject '_model.hdf5'],...
    '/best_param',...
    [5 pow_size(3) pow_size(2)]);

h5create([res_dir '/' subject '_model.hdf5'],...
    '/best_index',...
    [5 pow_size(3) pow_size(2)]);

h5create([res_dir '/' subject '_model.hdf5'],...
    '/best_mse',...
    [5 pow_size(3) pow_size(2)]);

for fold = 1:5
    
    train_indices = ismember(item, ui(~ismember(indices,fold)));
    test_indices = ismember(item, ui(ismember(indices,fold)));
    
    h5write([res_dir '/' subject '_model.hdf5'],'/train_indices',single(train_indices)',[fold 1],[1 length(train_indices)]);
    h5write([res_dir '/' subject '_model.hdf5'],'/test_indices',single(test_indices)',[fold 1],[1 length(test_indices)]);
    
    [B1, Y_hat, best_index, best_param, best_mse] = train_predict_ridge(pow_all,...
        pow_size, train_indices, X);
    
    h5write([res_dir '/' subject '_model.hdf5'],...
        '/model_weight',...
        single(reshape(B1,[1 size(B1,1) size(B1,2) size(B1,3) size(B1,4)])),...
        [fold 1 1 1 1],...
        [1 size(B1,1) size(B1,2) size(B1,3) size(B1,4)]);
        
        % fold by channel by freq by feature x penalty param
    
    h5write([res_dir '/' subject '_model.hdf5'],...
        '/best_index',...
        reshape(single(best_index),[1 size(best_index,1) size(best_index,2)]),...
        [fold 1 1],...
        [1 pow_size(3) pow_size(2)]);
    
    h5write([res_dir '/' subject '_model.hdf5'],...
        '/best_param',...
        reshape(single(best_param),[1 size(best_param,1) size(best_param,2)]),...
        [fold 1 1],...
        [1 pow_size(3) pow_size(2)]);

    h5write([res_dir '/' subject '_model.hdf5'],...
        '/best_mse',...
        reshape(single(best_mse),[1 size(best_mse,1) size(best_mse,2)]),...
        [fold 1 1],...
        [1 pow_size(3) pow_size(2)]);
    
end