function z = nanzscore( d, reference )
%NANZSCORE  zscore for data with missing values
%
%z = zscore( d )
% d is a m x n matrix of values
% z is a m x n matrix of standardized values (mean 0, std 1) computed
% independly for each column of d
% 
% z = zscore( d, reference )
% reference is a vector of integer indices less than m specifying a subset
% of rows in d that will be used to compute the mean and variance
% return z a m x n matrix of distances from each point (in standard units)
% from the reference sample
%
% Example
%   load carbig
%   X = [MPG Acceleration Weight Displacement];
%   nanzscore(X)
%
% See also mgrpcov, mah

if (nargin < 2 )
  [mu, v] = mgrpstats(d);
else
  [mu, v] = mgrpstats(d(reference,:));
end
z = scale( center(d,mu), sqrt(v));

function [means, v, gn, sx, sx2, labels] = mgrpstats( x, g )
%MGRPSTATS univariate summary statistics stratified by group
%
%[means, v, gn, sx, sx2, labels] = mgrpstats( x, g )
% x is a m*n matrix. nans are ignored
% g is a grouping variable of length m. it can be integer, char or cellstr
% means - mean for each group
% v     - variance for each group
% gn    - number of present observations for each groups
% sx    - sum of the xs for each group
% sx2   - sum of squares for each group
% labels - levels of grouping variable g in the same order as the above
%          statisitics
%
% Example
%  load carbig
%  X = [MPG Acceleration Weight Displacement];
%  [u,v,gn,sx,sx2,labels] = mgrpstats( X, Cylinders);
%  table( {'Cylinders' 'MPG' 'Acc', 'Weight', 'Disp'}, labels, u );
%
% See also mgrpcov

% $Id: mgrpstats.m,v 1.7 2006/12/26 22:54:11 Mike Exp $
% Copyright 2006 Mike Boedigheimer
% Amgen Inc.
% Department of Computational Biology
% mboedigh@amgen.com
% 

if ( nargin < 2 )
    idx = ones( size(x,1), 1);
    labels = 1;
else
    if isvector(g), g = g(:); end;
    [idx,labels] = grp2ind(g);
end
[m,n] = size(x);

%transpose x if it makes more sense
if ( m ~= length(idx) ) 
     error('linstats:mgrpstats:InvalidArgument', 'g must have the same number of elements as there are rows in x'); 
end;    

m     = length(labels);
sx    = nan( m, n );
sx2   = nan( m, n );
gn    = nan( m, n );


for i = 1:m
    j = idx == i;

    % copy of relevant rows of x
    tmp = x(j,:);
    % find nans
    k = isnan( tmp );
    % set to zero
    tmp(k) = 0;

    % calc sum
    sx(i,:)  = sum ( tmp,1 );
    
    % count number of elements that aren't NaN
    gn(i,:) = sum(~k,1);

    
    % calc sum x.*x
    sx2(i,:) = sum ( tmp.^2,1 );
end;

gn(gn == 0 ) = NaN;
means = sx./gn;

gn(gn==1) = NaN;
v = ( sx2 - (sx.*sx)./gn )./(gn-1);

function [d, V] = center(X, V, dim)
%CENTER subtracts a vector from each row of a matrix
%
%data utility to simplify subtracting each row of a matrix by a vector
%
% d = center(X, V, dim)
% X is a m x n matrix
% V is a 1 x n vector. defaults to column sum of X
% dim is a scalar equal to 1 or 2. 1 to subtract a row vector, 2
% to subtract a column vector
% d is a new matrix with each row of X divided by V
%
%Example
%   load carbig
%   X = [MPG Acceleration Weight Displacement];
%   z = center( X, nanmean(X) ); 
%   z = center(X, mean(X,2), 2);

% $Id: center.m,v 1.4 2006/12/26 22:54:06 Mike Exp $
% Copyright 2006 Mike Boedigheimer
% Amgen Inc.
% Department of Computational Biology
% mboedigh@amgen.com
% 

if nargin < 3
    dim = 1;
end;

if ( nargin < 2 )
    V = mean(X, dim);
end;


[m, n] = size(X);
s = n;
if dim == 2
    s = m;
end;

if ~isscalar(V) && (~isvector(V) || length(V) ~= s)
     error('linstats:scale:InvalidArgument', 'Input argument must be a vector');
end


if dim==2
    d = X - repmat(V, 1, n);
else
    d = X - repmat(V, m, 1);
end;
    
function d = scale(X, V, dim)
%SCALE divides a vector from each row in a matrix
%
% data utility to simplify dividing each row in a matrix by a vector
%
% d = scale(X, V)
% X is a m x n matrix
% V is a 1 x n vector. defaults to column sum of X
% d is a new matrix with each row of X divided by V
%
%Example
%   load carbig
%   X = [MPG Acceleration Weight Displacement];
%   z = scale( center( X, mean(X )), std(X)); %zscore
%   z = scale(X, sum(X,1), 2);
%
%See also center

% $Id: scale.m,v 1.6 2006/12/26 22:54:12 Mike Exp $
% Copyright 2006 Mike Boedigheimer
% Amgen Inc.
% Department of Computational Biology
% mboedigh@amgen.com
% 

if nargin < 3
    dim = 1;
end;

if nargin < 2
    V = sum(X, dim);
end

[m, n] = size(X);
s = n;
if dim == 2
    s = m;
end;

if ~isscalar(V) && (~isvector(V) || length(V) ~= s)
     error('linstats:scale:InvalidArgument', 'Input argument must be a vector');
end

if dim==2
    d = X./repmat(V, 1,n);
else
    d = X./repmat(V, m, 1);
end;
    