function varargout = tfce_ttest_onesample(ts,tails,nperm)
%TFCE_TTEST_ONESAMPLE performs maximal statistic permutation testing
%   [varargout] = tfce_ttest_onesample(ts,tails,nperm) corrects
%   a one-sample t-test (mean > 0) for multiple comparisons via a 
%   permutation procedure (random sign flipping). Maximal t-stats of
%   permuted data are compared with t-stats in the unshuffled data.
%
%   Arguments:
%   ts -- a 2D (t,subject) matrix of timeseries
%	tails -- 1 or 2 tailed test
%   nperm -- number of permutations to perform. More permutations yield
%   more precise correct p-values.
%   -- img the 3D image to be transformed
%   -- H height exponent
%   -- E extent exponent
%   -- C connectivity
%   -- ndh step number for cluster formation
%
%   Output:
%	If tails == 1:
%   pcorr --  map of corrected p-values
%	If tails == 2:
%	pcorr_pos -- corrected p-values for positive effects
%	pcorr_neg -- corrected p-values for negative effects

H = 2;
E = 0.5;
C = 4;
dh = .01;

% calculate matrix size
bsize = size(ts);
nsub = bsize(2);
bsize = bsize(1);

% set tranform function
if tails == 1
    transform = @tfce_transform;
else
    transform = @tfce_transform_twotailed;
end

% calculate true mean image
truestat = mean(ts,2)./(std(ts,0,2)./sqrt(nsub)); % true t
implicitmask = ~isnan(truestat);
tfcestat = transform(truestat,H,E,C,dh);


% p-values for comparison
tvals = tfcestat(implicitmask);
if tails == 2
	tvals=abs(tvals);
end
ntp = length(tvals);

% extract occupied timepoints for permutation test
occts = NaN(ntp,nsub);
for s = 1:nsub
    curts = ts(:,s);
    occts(:,s) = curts(implicitmask);
end

global parworkers
parworkers = 4;
pp = gcp('nocreate');
try 
    parworkers = pp.NumWorkers;
    parexist = 1;
    warning('Existing parallel pool discovered with %d workers. Overriding parworkers argument to use this pool.',parworkers);
catch
    parexist = 0;
    if parworkers > 0
        parpool(parworkers);
    end
end

% initialize progress indicator
parfor_progress(nperm);

% cycle through permutations
exceedances = zeros(ntp,1);
parfor(p = 1:nperm,parworkers)   
    % permute signs
    relabeling = randi([0,1],nsub,1);
    roccts = occts;
    for s = 1:nsub
        if relabeling(s) == 1;
            roccts(:,s) = -occts(:,s);
        end
    end
    
    % calculate permutation statistic
    rstats =  mean(roccts,2)./(std(roccts,0,2)./sqrt(nsub));
    rts = zeros(bsize,1);
    rts(implicitmask) = rstats;
    rts = transform(rts,H,E,C,dh);
    rstats = rts(implicitmask);
    if tails == 2
        rstats = abs(rstats);
    end
    
    % compare maxima to true statistic and increment as appropriate
    curexceeds = max(rstats) >= tvals;
    exceedances = exceedances + curexceeds;
    
    % update progress indicator (only does so 1 in 5 to minimize overhead)
    if ~randi([0 4]);
        parfor_progress;
    end
end

% create corrected p-value image
corrected = exceedances./nperm;
pcorr = ones(bsize,1);
pcorr(implicitmask) = corrected;

% split into positive and negative effects (if needed)
if tails == 2
	pos = truestat>0;
	pcorr_pos = pcorr;
	pcorr_pos(~pos) = 1;
	pcorr_neg = pcorr;
	pcorr_neg(pos) = 1;
end

% assign output to varargout
if tails == 1
	varargout{1} = pcorr;
else
	varargout{1} = pcorr_pos;
	varargout{2} = pcorr_neg;
end

end