function [spos_PM_ts, spos_AT_ts, cat_PM_ts, cat_AT_ts] = gen_reins_ts()
%GEN_REINS_TS runs stats and generates reinstatement figures for fig 3b
%
% INPUTS
% 
% none: this function reads intermediate data from the data dir
%
% OUTPUTS
%
% none: this function generates figures for display

% By James Kragel, 2021
% Distributed under Open Source BSD License

% reins_for_corr contains aucs from reinstatement model directory
% for all subjects, s: serial position, c: category, p: PM, a: AT
load('reins_auc', 'sp', 'sa', 'cp', 'ca');

% smooth raw auc timeseries
spos_PM_ts = smoother(sp);
spos_AT_ts = smoother(sa);
cat_PM_ts = smoother(cp);
cat_AT_ts = smoother(ca);

% and differences
spos_ts = smoother(sp-sa); %#ok<*NASGU>
cat_ts = smoother(cp-ca);

% perform non-parametric tests w/ tfce for stats vs. chance of 0.5
p_PM_spos = tfce_ttest_onesample(spos_PM_ts-.5, 1, 10000);
p_AT_spos = tfce_ttest_onesample(spos_AT_ts-.5, 1, 10000);
p_PM_cat = tfce_ttest_onesample(cat_PM_ts-.5, 1, 10000);
p_AT_cat = tfce_ttest_onesample(cat_AT_ts-.5, 1, 10000);

% more importantly, compare reinstatement between the two networks, greater
% tepmoral reinstatement within PM and category within AT
p_spos = tfce_ttest_onesample(spos_ts, 1, 10000);
p_cat = tfce_ttest_onesample(-cat_ts, 1, 10000);

% and organize data to plot the timeseries
x = -900:20:-120;
yat = nanmean(spos_AT_ts');
eat = nansem(spos_AT_ts');
ypm = nanmean(spos_PM_ts');
epm = nansem(spos_PM_ts');

figure; set(gcf,'Position',[99   125   454   811]);
subplot(2,1,1);
[hl, hp] = boundedline(x, [yat; ypm]', [eat; epm]');
hr = refline(0,.5); hr.Color = 'k'; hr.LineStyle = '--'; hr.LineWidth = 2;
uistack(hr,'bottom');
hl(1).LineWidth = 2; hl(2).LineWidth = 2;
hl(1).Color = [112 191 65]/255; hp(1).FaceColor =  [112 191 65]/255;
hl(2).Color = [119 63 155]/255; hp(2).FaceColor =  [119 63 155]/255;

hp(1).FaceAlpha = .2;
hp(2).FaceAlpha = .2;

set(gca,'YLim', [.48 .54],'box','off','LineWidth',2,'FontSize',16,...
    'XLim', [-900 -120]);
xlabel('Time (ms)');
ylabel({'Serial Position','Decoding (AUC)'});
hold on;
colors = [119 63 155;
    112 191 65;
    0 0 0]/255;

% add significance indicators

i = p_PM_spos<.05;
l = bwlabeln(i);
for i = 1:max(l)
    hp1 = plot(x(l==i),ypm(l==i),'-');
    hp1.Color = 'r'; %colors(1,:);
    hp1.LineWidth = 2;
    [~,p_toplot] = ttest(nanmean(spos_PM_ts(l==i,:)),.5);
    ht = text(x(find(l==i,1,'first')), .52, sprintf('p = %0.4f',p_toplot));
    ht.Color = 'r';
end

i = p_AT_spos<.05;
l = bwlabeln(i);
for i = 1:max(l)
    hp2 = plot(x(l==i),yat(l==i),'-');
    hp2.Color = 'r'; %colors(1,:);
    hp2.LineWidth = 2;
    [~,p_toplot] = ttest(nanmean(spos_AT_ts(l==i,:)),.5);
    ht = text(x(find(l==i,1,'first')), .51, sprintf('p = %0.4f',p_toplot));
    ht.Color = 'r';
end

i = p_spos<.05;
l = bwlabeln(i);

for i = 1:max(l)
    hp3 = plot(x(l==i),.53*ones(size(x(l==i))),'-');
    hp3.Color = 'k'; %colors(1,:);
    hp3.LineWidth = 2;
    [~,p_toplot] = ttest(nanmean(spos_AT_ts(l==i,:)),nanmean(spos_PM_ts(l==i,:)));
    ht = text(x(find(l==i,1,'first')), .535, sprintf('p = %0.4f',p_toplot));
    ht.Color = 'r';
end

yat = nanmean(cat_AT_ts'); %#ok<*UDIM>
eat = nansem(cat_AT_ts');
ypm = nanmean(cat_PM_ts');
epm = nansem(cat_PM_ts');

subplot(2,1,2);
[hl, hp] = boundedline(x, [yat; ypm]', [eat; epm]');
hr = refline(0,.5); hr.Color = 'k'; hr.LineStyle = '--'; hr.LineWidth = 2;
uistack(hr,'bottom');
hl(1).LineWidth = 2; hl(2).LineWidth = 2;
hl(1).Color = [112 191 65]/255; hp(1).FaceColor =  [112 191 65]/255;
hl(2).Color = [119 63 155]/255; hp(2).FaceColor =  [119 63 155]/255;

hp(1).FaceAlpha = .2;
hp(2).FaceAlpha = .2;

set(gca,'YLim', [.48 .54],'box','off','LineWidth',2,'FontSize',16,...
    'XLim', [-900 -120]);
xlabel('Time (ms)');
ylabel({'Category','Decoding (AUC)'});

hold on

i = p_PM_cat<.05;
l = bwlabeln(i);
for i = 1:max(l)
    hp1 = plot(x(l==i),ypm(l==i),'-');
    hp1.Color = 'r'; %colors(1,:);
    hp1.LineWidth = 2;
    [~,p_toplot] = ttest(nanmean(cat_PM_ts(l==i,:)),.5);
    ht = text(x(find(l==i,1,'first')), .52, sprintf('p = %0.4f',p_toplot));
    ht.Color = 'r';
end

i = p_AT_cat<.05;
l = bwlabeln(i);
for i = 1:max(l)
    hp2 = plot(x(l==i),yat(l==i),'-');
    hp2.Color = 'r'; %colors(1,:);
    hp2.LineWidth = 2;
    [~,p_toplot] = ttest(nanmean(cat_AT_ts(l==i,:)),.5);
    ht = text(x(find(l==i,1,'first')), .51, sprintf('p = %0.4f',p_toplot));
    ht.Color = 'r';
end

i = p_cat<.05;
l = bwlabeln(i);

for i = 1:max(l)
    hp3 = plot(x(l==i),.53*ones(size(x(l==i))),'-');
    hp3.Color = 'k'; %colors(1,:);
    hp3.LineWidth = 2;
    [~,p_toplot] = ttest(nanmean(cat_AT_ts(l==i,:)),nanmean(cat_PM_ts(l==i,:)));
    ht = text(x(find(l==i,1,'first')), .535, sprintf('p = %0.4f',p_toplot));
    ht.Color = 'r';
end

% output data, 3b - top
% xlswrite('SourceData.xlsx', {'Figure3bt'}, 'Figure3bt', 'A3');
% 
% xlswrite('SourceData.xlsx', {'Subject'}, 'Figure3bt', 'B6');
% xlswrite('SourceData.xlsx', [1:69]', 'Figure3bt', 'C6:C74');
% 
% xlswrite('SourceData.xlsx', {'Timepoint PM'}, 'Figure3bt', 'D4');
% xlswrite('SourceData.xlsx', {'Timepoint AT'}, 'Figure3bt', 'AR4');
% xlswrite('SourceData.xlsx', [1:40 1:40], 'Figure3bt', 'D5');
% 
% xlswrite('SourceData.xlsx', [spos_PM_ts; spos_AT_ts]', 'Figure3bt', 'D6');
% 
% 
% % output data, 3b - bottom
% 
% xlswrite('SourceData.xlsx', {'Figure3bb'}, 'Figure3bb', 'A3');
% 
% xlswrite('SourceData.xlsx', {'Subject'}, 'Figure3bb', 'B6');
% xlswrite('SourceData.xlsx', [1:69]', 'Figure3bb', 'C6:C74');
% 
% xlswrite('SourceData.xlsx', {'Timepoint PM'}, 'Figure3bb', 'D4');
% xlswrite('SourceData.xlsx', {'Timepoint AT'}, 'Figure3bb', 'AR4');
% xlswrite('SourceData.xlsx', [1:40 1:40], 'Figure3bb', 'D5');
% 
% xlswrite('SourceData.xlsx', [cat_PM_ts; cat_AT_ts]', 'Figure3bb', 'D6');

end

function ts = smoother(ts)

ts(:,all(isnan(ts)))=[];
% remove mean
ts_means = mean(ts);
ts = bsxfun(@minus, ts, ts_means);
% smooth
ts = smoothts(ts', 'g', 5, 1.5)';
% add mean
ts = bsxfun(@plus, ts, ts_means);

end
