function [owAT_idx, owPM_idx, obetween_idx] = matched_randsample(AT, PM, ...
    dist, bp_shared)
%MATCHED_RANDSAMPLE helper function to randomly select electrode pairs
%  matched for inter-electrode distance for connectivity analysis
%
% INPUTS
%
% AT: logical - boolean indicating whether a contact is within AT network
% PM: logical - boolean indicating whether a contact is within PM network
% dist: double - array specifying distance between contacts
% bp_shared: logical - array specifying whether the combination of bipolar
% pairs share a single individual contact (e.g., A1-A2, A2-A3)
%
% OUTPUTS
%
% owAT_idx: indices for matched within AT pairs
% owPM_idx: indices for matched within PM pairs
% obetween_idx: indices for matched between network pairs

% By James Kragel, 2021
% Distributed under Open Source BSD License


% make masks
[wPM_mask, wAT_mask, between_mask, mask] = make_masks(AT, PM, ...
    dist, bp_shared);

% build sampling distribution where distance is matched across all of the
% folds

dist_PM = dist(wPM_mask & mask);
dist_AT = dist(wAT_mask & mask);
dist_between = dist(between_mask & mask);

if sum(PM) < sum(AT)
    PM_min = true;
else
    PM_min = false;
end

wPM_idx = find(wPM_mask & mask);
wAT_idx = find(wAT_mask & mask);
wbetween_idx = find(between_mask & mask);

n_sample = length(wbetween_idx); %min(length(wPM_idx),length(wAT_idx));
owAT_idx = nan(1000,n_sample);
owPM_idx = owAT_idx;
obetween_idx = owPM_idx;

for n = 1:1000
    % build matched pairs to whichever pool of electrodes has the fewer
    % number of observations, and find draw pairs matching on distance
    
    % keep track of whether a pair was sampled, to ensure sampling without
    % replacement
    used_PM = false(size(wPM_idx));
    used_AT = false(size(wAT_idx));
    used_between = false(size(wbetween_idx));
    
    perm_idx = randperm(length(wbetween_idx));
    
    within_PM_idx = nan(size(perm_idx));
    within_AT_idx = nan(size(perm_idx));
    
    for s = 1:length(perm_idx)
        
        
        this_diff = dist_between(perm_idx(s));
        diff_within_AT = dist_AT - this_diff;
        diff_within_PM = dist_PM - this_diff;
        
        % this ordering seems somewhat relevant
        [ds,i] = sort(abs(diff_within_PM));
        matched_PM = false; PM_idx = [];
        for k = 1:length(i)
            if ~used_PM(i(k)) && (abs(diff_within_PM(i(k))) < 15) %15
                within_PM_idx(s) = i(k);
                used_PM(i(k)) = true;
                PM_idx = i(k);
                matched_PM = true;
                break
            end
        end
        
        [ds,i] = sort(abs(diff_within_AT));
        matched_AT = false;AT_idx = [];
        for k = 1:length(i)
            if ~used_AT(i(k)) && (abs(diff_within_AT(i(k))) < 15) %15
                within_AT_idx(s) = i(k);
                used_AT(i(k)) = true;
                AT_idx = i(k);
                matched_AT = true;
                break
            end
        end
        
        
        if ~matched_AT || ~matched_PM
            perm_idx(s) = nan;
            within_PM_idx(s) = nan;
            within_AT_idx(s) = nan;
            
            used_AT(AT_idx)=false;
            used_PM(PM_idx)=false;
        end
        
        
    end
    
    for c = 1:length(within_AT_idx)
        if ~isnan(within_AT_idx(c))
            owAT_idx(n,c) = wAT_idx(within_AT_idx(c));
        end
    end
    for c = 1:length(within_PM_idx)
        if ~isnan(within_PM_idx(c))
            owPM_idx(n,c) = wPM_idx(within_PM_idx(c));
        end
    end
    for c = 1:length(perm_idx)
        if ~isnan(perm_idx(c))
            obetween_idx(n,c) = wbetween_idx(perm_idx(c));
        end
    end
end

end

